#!/usr/bin/env/python3
"""Test pattoo configuration script."""

import os
import getpass
import grp
import unittest
import unittest.mock
import sys
import tempfile
from collections import defaultdict
from copy import deepcopy

import yaml

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(
        os.path.abspath(os.path.join(
            EXEC_DIR, os.pardir)), os.pardir)), os.pardir))
_EXPECTED = '''\
{0}pattoo-shared{0}tests{0}pattoo_shared_{0}installation'''.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from tests.libraries.configuration import UnittestConfig
from tests.libraries import general
from pattoo_shared.installation import configure
from pattoo_shared import files


class Test_Config(unittest.TestCase):
    """Checks all functions for the Pattoo config script."""

    @classmethod
    def setUpClass(cls):
        """Declare class attributes for Unittesting."""
        # Initialize key variables
        cls.modifications = {
            'pattoo_api_agentd': {
                'ip_bind_port': 60601,
                'ip_listen_address': '::1'},
            'pattoo_apid': {
                'ip_bind_port': 60602,
                'ip_listen_address': '::3'},
            'pattoo_db': {
                'db_hostname': 'localhost',
                'db_max_overflow': 100,
                'db_name': 'pattoo',
                'db_password': 'n5PaNcV85vR6jmW3',
                'db_username': 'pattoo'},
            'pattoo_ingesterd': {
                'ingester_interval': 3600,
                'multiprocessing': True}
        }

        cls.default = {
            'pattoo_api_agentd': {
                'api_encryption_email': 'test_api@example.org',
                'ip_bind_port': 20201,
                'ip_listen_address': '0.0.0.0'},
            'pattoo_apid': {
                'access_token_exp': '15_m',
                'ip_bind_port': 20202,
                'ip_listen_address': '0.0.0.0',
                'jwt_secret_key': '9uBqwWTc-02c2aaK99ULdQ',
                'refresh_token_exp': '1_D'},
            'pattoo_db': {
                'db_hostname': 'localhost',
                'db_max_overflow': 20,
                'db_name': 'pattoo',
                'db_password': 'password',
                'db_pool_size': 10,
                'db_username': 'pattoo'},
            'pattoo_ingesterd': {
                'batch_size': 500,
                'graceful_timeout': 10,
                'ingester_interval': 3600},
            'boo': 'boo'
        }

        cls.existing = {
            'pattoo_api_agentd': {
                'api_encryption_email': 'test_api@example.org',
                'ip_bind_port': 60601,
                'ip_listen_address': '::1'},
            'pattoo_apid': {
                'access_token_exp': '15_m',
                'ip_bind_port': 60602,
                'ip_listen_address': '::3',
                'jwt_secret_key': '9uBqwWTc-02c2aaK99ULdQ',
                'refresh_token_exp': '1_D'},
            'pattoo_db': {
                'db_hostname': 'localhost',
                'db_max_overflow': 100,
                'db_name': 'pattoo',
                'db_password': 'n5PaNcV85vR6jmW3',
                'db_pool_size': 10,
                'db_username': 'pattoo'},
            'pattoo_ingesterd': {
                'multiprocessing': True},
            }

        cls.expected = {
            'pattoo_api_agentd': {
                'api_encryption_email': 'test_api@example.org',
                'ip_bind_port': 60601,
                'ip_listen_address': '::1'},
            'pattoo_apid': {
                'access_token_exp': '15_m',
                'ip_bind_port': 60602,
                'ip_listen_address': '::3',
                'jwt_secret_key': '9uBqwWTc-02c2aaK99ULdQ',
                'refresh_token_exp': '1_D'},
            'pattoo_db': {
                'db_hostname': 'localhost',
                'db_max_overflow': 100,
                'db_name': 'pattoo',
                'db_password': 'n5PaNcV85vR6jmW3',
                'db_pool_size': 10,
                'db_username': 'pattoo'},
            'pattoo_ingesterd': {
                'batch_size': 500,
                'graceful_timeout': 10,
                'ingester_interval': 3600,
                'multiprocessing': True},
            'boo': 'boo'
            }

    def test__merge_config(self):
        """Unittest to test the _merge_config function."""
        # Initialize key variables
        default = deepcopy(self.default)
        modifications = deepcopy(self.modifications)
        expected = deepcopy(self.expected)
        new_expected = deepcopy(expected)

        # Test
        result = configure._merge_config(default, modifications)
        self.assertTrue(isinstance(result, dict))
        self.assertFalse(isinstance(result, defaultdict))
        self.assertEqual(sorted(result), sorted(expected))

        # Make sure there are no defaultdict objects in the result.
        # yaml.safe_dump doesn't work correctly if defaultdicts are present.
        for _, value in result.items():
            self.assertFalse(isinstance(value, defaultdict))

        # Test with key not found in default
        modifications['test'] = 'test'
        new_expected['test'] = 'test'
        result = configure._merge_config(default, modifications)
        self.assertTrue(isinstance(result, dict))
        self.assertFalse(isinstance(result, defaultdict))
        self.assertEqual(sorted(result), sorted(new_expected))

        # Make sure there are no defaultdict objects in the result.
        # yaml.safe_dump doesn't work correctly if defaultdicts are present.
        for _, value in result.items():
            self.assertFalse(isinstance(value, defaultdict))

    def test_read(self):
        """Unittest to test the read method."""
        # Initialize key variables
        default = deepcopy(self.default)
        expected = deepcopy(self.expected)
        existing = deepcopy(self.existing)

        # Create temporary directory using the temp file package
        with tempfile.TemporaryDirectory() as temp_dir:
            file_path = os.path.join(temp_dir, 'pattoo_temp_config.yaml')

            # Delete file if already exists
            if os.path.exists(file_path) is True:
                os.remove(file_path)
            self.assertFalse(os.path.isfile(file_path))

            # Create configuration file
            with open(file_path, 'w') as fh_:
                yaml.safe_dump(existing, stream=fh_, default_flow_style=False)
            config = configure._Config(file_path, existing)
            result = config.read()
            self.assertEqual(result, existing)

            # Test updating the configuration
            config = configure._Config(file_path, default)
            result = config.read()
            for key, value in result.items():
                self.assertEqual(expected[key], value)
            self.assertEqual(sorted(result), sorted(expected))

    def test_update(self):
        """Test the update method."""
        # Initialize key variables
        default = deepcopy(self.default)
        expected = deepcopy(self.expected)
        existing = deepcopy(self.existing)
        filename = 'pattoo_temp_config'
        key = 'pattoo_apid'

        self.maxDiff = None

        # Test with no existing configuration
        with tempfile.TemporaryDirectory() as temp_dir:
            # Configuration file should not exist
            file_path = os.path.join(temp_dir, '{}.yaml'.format(filename))
            self.assertFalse(os.path.isfile(file_path))

            # Test
            config = configure._Config(file_path, default)
            config.update()
            self.assertTrue(os.path.isfile(file_path))

            # Test to make sure the configuration was created
            result = config.read()
            self.assertEqual(result, default)

        # Test with existing configuration
        with tempfile.TemporaryDirectory() as temp_dir:
            # Configuration file should not exist
            file_path = os.path.join(temp_dir, '{}.yaml'.format(filename))
            self.assertFalse(os.path.isfile(file_path))

            # Create configuration file
            with open(file_path, 'w') as fh_:
                yaml.safe_dump(
                    existing, stream=fh_, default_flow_style=False)

            # Configuration file should exist
            self.assertTrue(os.path.isfile(file_path))

            # Test
            config = configure._Config(file_path, default)
            config.update()
            with open(file_path, 'r') as fh_:
                result = yaml.safe_load(fh_.read())

            # self.assertEqual(result, existing)
            self.assertEqual(result, expected)

        # Test with existing configuration with directory
        with tempfile.TemporaryDirectory() as temp_dir:
            # Configuration file should not exist
            file_path = os.path.join(temp_dir, '{}.yaml'.format(filename))
            self.assertFalse(os.path.isfile(file_path))

            # Create and delete a temporary directory
            new_dir = tempfile.mkdtemp()
            os.rmdir(new_dir)
            self.assertFalse(os.path.isdir(new_dir))

            # Update the configuration
            new_default = deepcopy(default)
            new_expected = deepcopy(expected)
            new_default[key]['dummy_directory'] = new_dir
            new_expected[key]['dummy_directory'] = new_dir

            # Create configuration file
            with open(file_path, 'w') as fh_:
                yaml.safe_dump(
                    existing, stream=fh_, default_flow_style=False)

            # Configuration file should exist
            self.assertTrue(os.path.isfile(file_path))

            # Test
            config = configure._Config(file_path, new_default)
            config.update()
            with open(file_path, 'r') as fh_:
                result = yaml.safe_load(fh_.read())
            self.assertEqual(result, new_expected)

            # Verify the creation of the new directory
            self.assertTrue(os.path.isdir(new_dir))
            os.rmdir(new_dir)
            self.assertFalse(os.path.isdir(new_dir))

    def test_validate(self):
        """Test the update method."""
        # Initialize key variables
        key = 'pattoo_apid'
        default = deepcopy(self.default)
        existing = deepcopy(self.existing)
        existing.pop(key)

        # Create temporary directory using the temp file package
        with tempfile.TemporaryDirectory() as temp_dir:
            file_path = os.path.join(temp_dir, 'pattoo_temp_config.yaml')

            # Delete file if already exists
            if os.path.exists(file_path) is True:
                os.remove(file_path)
            self.assertFalse(os.path.isfile(file_path))

            # Create configuration file
            with open(file_path, 'w') as fh_:
                yaml.safe_dump(existing, stream=fh_, default_flow_style=False)

            # Test, nothing should happen
            config = configure._Config(file_path, default)
            config.validate()

    def test__create_directories(self):
        """Unittest to test the _create_directories function."""
        # Initialize key variables
        key = 'pattoo_apid'
        default = deepcopy(self.default)
        directory = tempfile.mkdtemp()
        os.rmdir(directory)
        default[key]['test_directory'] = directory

        # Test
        self.assertFalse(os.path.isdir(directory))
        configure._create_directories(default)
        self.assertTrue(os.path.isdir(directory))
        os.rmdir(directory)
        self.assertFalse(os.path.isdir(directory))

    def test_configure_component(self):
        """Unittest to test the configure_component function."""
        # Initialize key variables
        test_name = general.random_string()
        expected = {
            'polling_interval': 300,
        }
        with tempfile.TemporaryDirectory() as temp_dir:
            file_path = os.path.join(temp_dir, '{}.yaml'.format(test_name))

            # Create config file
            configure.configure_component(test_name, temp_dir, expected)

            # Retrieve config dict from yaml file
            config = configure._Config(file_path, expected)
            result = config.read()
            self.assertEqual(result, expected)


class TestUserFunctions(unittest.TestCase):
    """Checks all functions for the Pattoo config script."""

    def test_create_user(self):
        """Unittest to test the create_user function."""
        pass

    def test_group_exists(self):
        """Unittest to test the group_exists function."""
        # Test case for when the group does not exist
        with self.subTest():
            expected = False

            # Generating random string
            result = configure.group_exists(general.random_string())
            self.assertEqual(result, expected)

        # Test case for when the group exists
        with self.subTest():
            expected = True

            # Creating grp.struct object
            grp_struct = grp.getgrgid(os.getgid())
            result = configure.group_exists(grp_struct.gr_name)
            self.assertEqual(result, expected)

    def test_user_exists(self):
        """Unittest to test the user_exists function."""
        # Test case for when the user does not exist
        with self.subTest():
            expected = False

            # Generating random string
            result = configure.user_exists(general.random_string())
            self.assertEqual(result, expected)

        # Test case for when the user does exist
        with self.subTest():
            expected = True
            result = configure.user_exists(getpass.getuser())
            self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



"""Functions for setting up a virtual environment for the installation."""
import os
import getpass
from pattoo_shared.installation import shared, configure
from pattoo_shared import log


class Venv():
    """Virtual Environment class."""

    def __init__(self, directory):
        """Initialize the class.

        Args:
            directory: Activation directory

        Returns:
            None

        """
        # Initialize key variables
        self._directory = directory
        self._path = os.environ.get('PATH')

    def create(self):
        """Create virtual environment for pattoo installation.

        Args:
            None

        Returns:
            None

        """
        # Say what we're doing
        print('??: Create virtual environment')
        command = 'python3 -m virtualenv {}'.format(self._directory)
        shared.run_script(command)
        print('OK: Virtual environment created')
        # Ensure venv is owned by pattoo if the pattoo user exists
        if configure.user_exists(
                'pattoo') and configure.group_exists('pattoo'):
            if getpass.getuser() == 'root':
                shared.run_script(
                    'chown -R pattoo:pattoo {}'.format(self._directory))

    def activate(self):
        """Activate the virtual environment in the current interpreter.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        filepath = os.path.join(self._directory, 'bin/activate_this.py')

        # Open activte_this.py for reading
        try:
            f_handle = open(filepath)
        except PermissionError:
            log.log2die_safe(1061, '''\
Insufficient permissions on file: {}. Ensure the directory has the necessary \
read-write permissions'''.format(filepath))
        except FileNotFoundError:
            log.log2die_safe(1074, '''\
activate_this.py for the venv has not been created at {} with virtualenv. \
Ensure that its created using the module "virtualenv"
'''.format(filepath))
        else:
            with f_handle:
                code = compile(f_handle.read(), filepath, 'exec')
                exec(code, dict(__file__=filepath))

    def deactivate(self):
        """Deactivate the virtual environment in the current interpreter.

        Args:
            None

        Returns:
            None

        """
        # Reset the 'PATH' variable
        os.environ['PATH'] = self._path

        # Delete the 'VIRTUAL_ENV' reference
        venv = os.environ.get('VIRTUAL_ENV')
        if bool(venv) is True:
            del os.environ['VIRTUAL_ENV']


class PIPpath():
    """Set PIP paths for installations."""

    def __init__(self, directory):
        """Initialize the class.

        Args:
            directory: Virtual Environment directory

        Returns:
            None

        """
        # Initialize key variables
        self._directory = directory
        self._path = os.environ.get('PATH')

    def set(self):
        """Set path to view PIP packages installed in venv.

        Args:
            None

        Returns:
            None

        """
        # Set the 'PATH' variable
        os.environ['PATH'] = '{}:{}'.format(self._directory, self._path)

    def reset(self):
        """Set path to view PIP packages installed in venv.

        Args:
            None

        Returns:
            None

        """
        # Reset the 'PATH' variable
        os.environ['PATH'] = self._path


def environment_setup(directory):
    """Create and activate virtual environment.

    Args:
        directory: The path to the virtual environment

    Returns:
        venv: Venv instance

    """
    # Setup the environment
    venv = Venv(directory)
    venv.create()
    venv.activate()
    return venv



#!/usr/bin/env python3
"""Test the phttp module."""

# Standard imports
import json
import hashlib
import uuid
import os
import random
import tempfile
import sys
from time import time
import unittest
from unittest.mock import patch

# PIP imports
import requests_mock

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import phttp
from pattoo_shared import data
from pattoo_shared import converter
from pattoo_shared import files
from pattoo_shared import encrypt
from tests.libraries.configuration import UnittestConfig
from tests.libraries import general as ta


class Test_Post(unittest.TestCase):
    """Test _Post."""

    # Create agent data
    agentdata = ta.test_agent()

    # Get agent variables
    identifier = agentdata.agent_id
    _data = converter.agentdata_to_post(agentdata)
    data = converter.posting_data_points(_data)

    def test___init__(self):
        """Testing method or function named __init__."""

        # Test
        _post = phttp._Post(self.identifier, self.data)

        self.assertEqual(_post._identifier, self.identifier)
        self.assertEqual(_post._data, self.data)


class TestPost(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    # Create agent data
    agentdata = ta.test_agent()

    # Get agent variables
    identifier = agentdata.agent_id
    _data = converter.agentdata_to_post(agentdata)
    data = converter.posting_data_points(_data)

    # Change tuple to list in the data
    _mod_data = json.dumps(data)
    mod_data = json.loads(_mod_data)

    def test___init__(self):
        """Testing method or function named __init__."""

        # Initialize
        post = phttp.Post(self.identifier, self.data)

        # Test
        expected_server_url = \
            '''http://127.0.0.6:50505/pattoo/api/v1/agent/receive/{}'''\
            .format(self.identifier)
        result_server_url = post._url

        self.assertEqual(result_server_url, expected_server_url)

    def test_post(self):
        """Testing method or function named post."""
        # Initialize
        post_test = phttp.Post(self.identifier, self.data)

        # Magically simulate post request
        with patch('pattoo_shared.phttp.requests.post') as mock_post:

            # Magically assign post response values
            mock_post.return_value.ok = True
            mock_post.return_value.text = 'OK'
            mock_post.return_value.status_code = 200

            # Run post
            success = post_test.post()

            # Check that the post request was to the right URL
            # and that it contained the right data
            mock_post.assert_called_with(
                '''http://127.0.0.6:50505/pattoo/api/v1/agent/receive/{}'''
                .format(self.identifier), json=self.data
                )

            # Assert that the success is True
            self.assertTrue(success)

    def test_purge(self):
        """Testing method or function named purge."""
        # Initialize
        purge_test = phttp.Post(self.identifier, self.data)

        # Magically simulate post request
        with patch('pattoo_shared.phttp.requests.post') as mock_post:

            # Magically assign post response values
            mock_post.return_value.ok = True
            mock_post.return_value.text = 'OK'
            mock_post.return_value.status_code = 200

            # Save data to cache
            phttp._save_data(self.data, self.identifier)

            # Run post
            purge_test.purge()

            # Check that the post request was to the right URL
            # and that it contained the right data
            mock_post.assert_called_with(
                '''http://127.0.0.6:50505/pattoo/api/v1/agent/receive/{}'''
                .format(self.identifier), json=self.mod_data
            )


class TestEncryptedPost(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    # Create agent data
    agentdata = ta.test_agent()

    # Get agent variables
    identifier = agentdata.agent_id
    _data = converter.agentdata_to_post(agentdata)
    data = converter.posting_data_points(_data)

    # Variables that will be modified by callback functions
    agent_publickey = None
    agent_email = None
    symmetric_key = None
    nonce = None

    # Initialize encrytion keys
    encrypt_agt = encrypt.Encryption(
        hashlib.md5('{}'.format(random.random()).encode()).hexdigest(),
        tempfile.mkdtemp()
    )
    encrypt_api = encrypt.Encryption(
        hashlib.md5('{}'.format(random.random()).encode()).hexdigest(),
        tempfile.mkdtemp()
    )

    # Create EncryptedPost object
    encrypted_post = phttp.EncryptedPost(identifier, data, encrypt_agt)

    def test___init__(self):
        """Testing method or function named __init__."""
        # Test variables
        pass

    def test_post(self):
        """Test EncryptedPost's post"""

        # Define callback functions

        # Key exchange callback for post request to process key exchange
        def exchange_post_callback(request, context):
            """Exchange callback for post request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)

            self.agent_publickey = json_dict['pattoo_agent_key']
            self.agent_email = json_dict['pattoo_agent_email']

            # Import agent public key
            self.encrypt_api.pimport(self.agent_publickey)

            # Trust public keys to enable encryption with traded keys
            agent_fp = self.encrypt_api.fingerprint(self.agent_email)
            self.encrypt_api.trust(agent_fp)

            # Send accepted response
            context.status_code = 202
            return 'Noted'

        # Key exchange callback for post request to process key exchange
        def exchange_get_callback(request, context):
            """Exchange callback for post request mock"""
            # Status is OK
            context.status_code = 200
            # Generate nonce
            self.nonce = hashlib.sha256(str(uuid.uuid4()).encode()).hexdigest()

            # Prepare API info to send to agent
            api_publickey = self.encrypt_api.pexport()

            # Encrypt nonce
            encrypted_nonce = self.encrypt_api.encrypt(
                self.nonce, self.encrypt_agt.fingerprint())

            json_response = {
                'api_email': self.encrypt_api.email,
                'api_key': api_publickey,
                'encrypted_nonce': encrypted_nonce
            }

            # Send data
            return json_response

        # Validation callback to respond to agent validation post request
        def validation_callback(request, context):
            """Validation callback for request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_nonce = json_dict['encrypted_nonce']
            encrypted_sym_key = json_dict['encrypted_sym_key']

            # Validate by decrypting the encrypted symmetric key
            # then using the symmetric key to decrypt the nonce
            # and check if it is the same as the one that was sent
            symmetric_key = self.encrypt_api.decrypt(encrypted_sym_key)
            nonce = self.encrypt_api.sdecrypt(encrypted_nonce, symmetric_key)

            if nonce == self.nonce:
                self.symmetric_key = symmetric_key
                context.status_code = 200
            else:
                context.status_code = 409

            return 'Result'

        # Encrypted post callback to respond to agent encrypted data
        def encrypted_callback(request, context):
            """Encrypted post callback for request mock"""
            # Retrieve encrypted data from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_data = json_dict['encrypted_data']
            # Decrypt data
            decrypted_data = self.encrypt_api.sdecrypt(
                encrypted_data, self.symmetric_key)
            # Unload
            data_dict = json.loads(decrypted_data)
            recv_data = data_dict['data']

            # Check that decrypted data is the same as the received
            # The two dictionaries are hashed then the values are compared
            agent_data = self.data
            agent_hash = hashlib.sha256(
                str(json.dumps(agent_data)).encode()).hexdigest()
            recv_hash = hashlib.sha256(
                str(json.dumps(recv_data)).encode()).hexdigest()
            if agent_hash == recv_hash:
                # Data received and decrypted successfully
                context.status_code = 202
            else:
                # Decryption failed
                context.status_code = 409

            return 'Noted'

        # Mock each requests
        with requests_mock.Mocker() as mock_:
            # Mock agent sending info to API server
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                text=exchange_post_callback)
            # Mock agent receiving API info
            mock_.get(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                json=exchange_get_callback)
            # Mock agent validation
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/validation',
                text=validation_callback
            )

            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/encrypted',
                text=encrypted_callback
            )

            # Run function
            success = self.encrypted_post.post()

            # Test
            self.assertTrue(success)

    def test_purge(self):
        """Test EncryptedPost's purge"""

        # Define callback functions

        # Key exchange callback for post request to process key exchange
        def exchange_post_callback(request, context):
            """Exchange callback for post request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)

            self.agent_publickey = json_dict['pattoo_agent_key']
            self.agent_email = json_dict['pattoo_agent_email']

            # Import agent public key
            self.encrypt_api.pimport(self.agent_publickey)
            # Trust public keys to enable encryption with traded keys
            agent_fp = self.encrypt_api.fingerprint(self.agent_email)
            self.encrypt_api.trust(agent_fp)

            # Send accepted response
            context.status_code = 202
            return 'Noted'

        # Key exchange callback for post request to process key exchange
        def exchange_get_callback(request, context):
            """Exchange callback for post request mock"""
            # Status is OK
            context.status_code = 200
            # Generate nonce
            self.nonce = hashlib.sha256(str(uuid.uuid4()).encode()).hexdigest()

            # Prepare API info to send to agent
            api_publickey = self.encrypt_api.pexport()

            # Encrypt nonce
            encrypted_nonce = self.encrypt_api.encrypt(
                self.nonce, self.encrypt_agt.fingerprint())

            # Create a json response
            json_response = {
                'api_email': self.encrypt_api.email,
                'api_key': api_publickey,
                'encrypted_nonce': encrypted_nonce
            }

            # Send data
            return json_response

        # Validation callback to respond to agent validation post request
        def validation_callback(request, context):
            """Validation callback for request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_nonce = json_dict['encrypted_nonce']
            encrypted_sym_key = json_dict['encrypted_sym_key']

            # Validate by decrypting the encrypted symmetric key
            # then using the symmetric key to decrypt the nonce
            # and check if it is the same as the one that was sent
            symmetric_key = self.encrypt_api.decrypt(encrypted_sym_key)
            nonce = self.encrypt_api.sdecrypt(encrypted_nonce, symmetric_key)

            if nonce == self.nonce:
                self.symmetric_key = symmetric_key
                context.status_code = 200
            else:
                context.status_code = 409

            return 'Result'

        # Encrypted post callback to respond to agent encrypted data
        def encrypted_callback(request, context):
            """Encrypted post callback for request mock"""
            # Retrieve encrypted data from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_data = json_dict['encrypted_data']
            # Decrypt data
            decrypted_data = self.encrypt_api.sdecrypt(
                encrypted_data, self.symmetric_key)
            # Unload
            data_dict = json.loads(decrypted_data)
            recv_data = data_dict['data']

            # Check that decrypted data is the same as the received
            # The two dictionaries are hashed then the values are compared
            agent_data = self.data
            agent_hash = hashlib.sha256(str(
                json.dumps(agent_data)).encode()).hexdigest()
            recv_hash = hashlib.sha256(str(
                json.dumps(recv_data)).encode()).hexdigest()
            if agent_hash == recv_hash:
                # Data received and decrypted successfully
                context.status_code = 202
            else:
                # Decryption failed
                context.status_code = 409

            return 'Noted'

        # Mock each requests
        with requests_mock.Mocker() as mock_:
            # Mock agent sending info to API server
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                text=exchange_post_callback)
            # Mock agent receiving API info
            mock_.get(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                json=exchange_get_callback)
            # Mock agent validation
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/validation',
                text=validation_callback
            )

            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/encrypted',
                text=encrypted_callback
            )

            # Save data to cache
            phttp._save_data(self.data, self.identifier)

            # Run purge
            self.encrypted_post.purge()

            # Check that URL's were called
            self.assertEqual(mock_.call_count, 4)


class TestPassiveAgent(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method or function named __init__."""
        pass

    def test_relay(self):
        """Testing method or function named relay."""
        pass

    def test_get(self):
        """Testing method or function named get."""
        pass


class TestEncryptedPostAgent(unittest.TestCase):
    """Test EncryptedPostAgent"""

    # Initialize key variables
    encrypt_agt = encrypt.Encryption(
        hashlib.md5('{}'.format(random.random()).encode()).hexdigest(),
        tempfile.mkdtemp()
    )
    encrypt_api = encrypt.Encryption(
        hashlib.md5('{}'.format(random.random()).encode()).hexdigest(),
        tempfile.mkdtemp()
    )

    # Variables that will be modified by callback functions
    agent_publickey = None
    agent_email = None
    symmetric_key = None
    nonce = None

    def test_agent(self):
        """Test agent post and purge"""

        # Create agent data
        agentdata = ta.test_agent()

        # Get agent variables
        _data = converter.agentdata_to_post(agentdata)
        data2post = converter.posting_data_points(_data)

        # Create agent
        encrypted_agent = phttp.EncryptedPostAgent(agentdata, self.encrypt_agt)

        # Define callback functions

        # Key exchange callback for post request to process key exchange
        def exchange_post_callback(request, context):
            """Exchange callback for post request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)

            self.agent_publickey = json_dict['pattoo_agent_key']
            self.agent_email = json_dict['pattoo_agent_email']

            # Import agent public key
            self.encrypt_api.pimport(self.agent_publickey)
            # Trust public keys to enable encryption with traded keys
            agent_fp = self.encrypt_api.fingerprint(self.agent_email)
            self.encrypt_api.trust(agent_fp)

            # Send accepted response
            context.status_code = 202
            return 'Noted'

        # Key exchange callback for post request to process key exchange
        def exchange_get_callback(request, context):
            """Exchange callback for post request mock"""
            # Status is OK
            context.status_code = 200
            # Generate nonce
            self.nonce = hashlib.sha256(str(uuid.uuid4()).encode()).hexdigest()

            # Prepare API info to send to agent
            api_publickey = self.encrypt_api.pexport()

            # Encrypt nonce
            encrypted_nonce = self.encrypt_api.encrypt(
                self.nonce, self.encrypt_agt.fingerprint())

            json_response = {
                'api_email': self.encrypt_api.email,
                'api_key': api_publickey,
                'encrypted_nonce': encrypted_nonce
            }

            # Send data
            return json_response

        # Validation callback to respond to agent validation post request
        def validation_callback(request, context):
            """Validation callback for request mock"""
            # Retrieve agent info from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_nonce = json_dict['encrypted_nonce']
            encrypted_sym_key = json_dict['encrypted_sym_key']

            # Validate by decrypting the encrypted symmetric key
            # then using the symmetric key to decrypt the nonce
            # and check if it is the same as the one that was sent
            symmetric_key = self.encrypt_api.decrypt(encrypted_sym_key)
            nonce = self.encrypt_api.sdecrypt(encrypted_nonce, symmetric_key)

            if nonce == self.nonce:
                self.symmetric_key = symmetric_key
                context.status_code = 200
            else:
                context.status_code = 409

            return 'Result'

        # Encrypted post callback to respond to agent encrypted data
        def encrypted_callback(request, context):
            """Encrypted post callback for request mock"""
            # Retrieve encrypted data from received request object
            json_data = request.json()
            json_dict = json.loads(json_data)
            encrypted_data = json_dict['encrypted_data']

            # Decrypt data
            decrypted_data = self.encrypt_api.sdecrypt(
                encrypted_data, self.symmetric_key)

            # Unload
            data_dict = json.loads(decrypted_data)
            recv_data = data_dict['data']

            # Check that decrypted data is the same as the received
            # The two dictionaries are hashed then the values are compared
            agent_data = data2post
            if len(agent_data) == len(recv_data):
                # Data received and decrypted successfully
                context.status_code = 202
            else:
                # Decryption failed
                context.status_code = 409

            return 'Noted'

        # Mock each requests
        with requests_mock.Mocker() as mock_:
            # Mock agent sending info to API server
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                text=exchange_post_callback)

            # Mock agent receiving API info
            mock_.get(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/key',
                json=exchange_get_callback)

            # Mock agent validation
            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/validation',
                text=validation_callback
            )

            mock_.post(
                'http://127.0.0.6:50505/pattoo/api/v1/agent/encrypted',
                text=encrypted_callback
            )

            # Run function
            success = encrypted_agent.post()

            # Test
            self.assertTrue(success)

            # Get agent identifier from data
            identifier = agentdata.agent_id

            # Save data to cache
            phttp._save_data(data2post, identifier)

            # Encrypted purge
            encrypted_agent.purge()

            # Check that both the post and purge exchanged keys and
            # send data
            self.assertEqual(mock_.call_count, 8)


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_post(self):
        """Testing method or function named post."""
        pass

    def test_purge(self):
        """Testing method or function named purge."""
        pass

    def test__save_data(self):
        """Testing method or function named _save_data."""
        # Initialize key variables
        identifier = data.hashstring(str(time()))

        # Test valid
        _data = {'Test': 'data'}
        success = phttp._save_data(_data, identifier)
        self.assertTrue(success)

        # Test invalid
        _data = ''
        success = phttp._save_data(_data, identifier)
        self.assertTrue(success)

    def test__log(self):
        """Testing method or function named _log."""
        pass


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from tests.libraries.configuration import UnittestConfig

from pattoo_shared.constants import DATA_FLOAT
from pattoo_shared.constants import DATA_INT
from pattoo_shared.constants import DATA_COUNT64
from pattoo_shared.constants import DATA_COUNT
from pattoo_shared.constants import DATA_STRING
from pattoo_shared.constants import DATA_NONE
from pattoo_shared.constants import MAX_KEYPAIR_LENGTH
from pattoo_shared.constants import DATAPOINT_KEYS
from pattoo_shared.constants import RESERVED_KEYS
from pattoo_shared.constants import CACHE_KEYS


from pattoo_shared.constants import PATTOO_WEB_SITE_PREFIX
from pattoo_shared.constants import PATTOO_API_SITE_PREFIX
from pattoo_shared.constants import PATTOO_API_AGENT_PREFIX


class TestConstants(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_constants(self):
        """Testing constants."""
        # Test data type constants
        self.assertEqual(DATA_FLOAT, 101)
        self.assertEqual(DATA_INT, 99)
        self.assertEqual(DATA_COUNT64, 64)
        self.assertEqual(DATA_COUNT, 32)
        self.assertEqual(DATA_STRING, 2)
        self.assertEqual(DATA_NONE, None)
        self.assertEqual(MAX_KEYPAIR_LENGTH, 512)
        self.assertEqual(
            DATAPOINT_KEYS,
            ('pattoo_checksum', 'pattoo_metadata', 'pattoo_data_type',
             'pattoo_key', 'pattoo_value', 'pattoo_timestamp'))
        self.assertEqual(
            RESERVED_KEYS,
            ('pattoo_checksum', 'pattoo_metadata', 'pattoo_data_type',
             'pattoo_key', 'pattoo_value', 'pattoo_timestamp',
             'pattoo_agent_id', 'pattoo_agent_polled_target',
             'pattoo_agent_program', 'pattoo_agent_hostname',
             'pattoo_agent_polling_interval'))
        self.assertEqual(
            CACHE_KEYS,
            ('pattoo_agent_id', 'pattoo_datapoints',
             'pattoo_agent_polling_interval', 'pattoo_agent_timestamp'))

        # Test pattoo API constants
        self.assertEqual(
            PATTOO_WEB_SITE_PREFIX, '/pattoo')

        # Test pattoo API constants
        self.assertEqual(
            PATTOO_API_SITE_PREFIX, '/pattoo/api/v1')
        self.assertEqual(
            PATTOO_API_AGENT_PREFIX, '{}/agent'.format(PATTOO_API_SITE_PREFIX))


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo times library."""

import time

# Pattoo libraries
from pattoo_shared import data
from pattoo_shared import log


def validate_timestamp(timestamp, polling_interval):
    """Validate timestamp to be a multiple of 'polling_interval' seconds.

    Args:
        timestamp: epoch timestamp in seconds
        polling_interval: Polling interval for data

    Returns:
        valid: True if valid

    """
    # Initialize key variables
    valids = []
    valid = False

    # Evaluate validity of the parameters
    valids.append(bool(polling_interval))
    valids.append(data.is_numeric(timestamp))
    valids.append(data.is_numeric(polling_interval))
    valids.append(isinstance(polling_interval, (int, float)))

    # Process data
    if False not in valids:
        test = (int(timestamp) // polling_interval) * polling_interval
        if test == timestamp:
            valid = True

    # Return
    return valid


def normalized_timestamp(_pi, timestamp=None):
    """Normalize timestamp to a multiple of 'polling_interval' seconds.

    Args:
        timestamp: epoch style timestamp in millseconds
        _pi: Polling interval for data in milliseconds. Defaults to assuming
            1000 if 'None'.

    Returns:
        value: Normalized value

    """
    # Initialize key variables
    if bool(_pi) is True:
        if isinstance(_pi, int) is False:
            log_message = ('''\
Invalid non-integer "polling_interval" value of {}'''.format(_pi))
            log.log2die(1029, log_message)
        else:
            polling_interval = abs(_pi)
    else:
        # Don't allow 0 values for polling_interval
        polling_interval = 1000

    # Process data
    if (timestamp is None) or (data.is_numeric(timestamp) is False):
        value = (
            int(time.time() * 1000) // polling_interval) * polling_interval
    else:
        value = (int(timestamp) // polling_interval) * polling_interval

    # Return
    return value


def timestamps(ts_start_raw, ts_stop_raw, _pi):
    """Get normalized timestamps list for every sampling_rate number of steps.

    Args:
        ts_start_raw: Timestamp of the start of the report
        ts_stop_raw: Timestamp of the end of the report
        _pi: Polling interval

    Returns:
        _timestamps: list of timstamps

    """
    # Normalize timestamps
    ts_start = normalized_timestamp(_pi, ts_start_raw)
    ts_stop = normalized_timestamp(_pi, ts_stop_raw)

    # Create a list of timestamps
    _timestamps = list(range(ts_start, ts_stop + _pi, _pi))

    # Returns
    return _timestamps



#!/usr/bin/env python3
"""Pattoo HTTP data classes."""

# Standard libraries
import os
import sys
import json
import urllib
import collections
from time import time

# pip3 libraries
import requests

# Pattoo libraries
from pattoo_shared import log
from pattoo_shared.configuration import Config
from pattoo_shared import converter
from pattoo_shared import encrypt

# Save items needed for encrypted purging inside a named tuple
EncryptionSuite = collections.namedtuple(
    'EncryptionSuite',
    'encrypted_post encryption symmetric_key session')

_KeyExchange = collections.namedtuple(
    '_KeyExchange',
    'encryption session key_exchange_url symmetric_key_url symmetric_key')

_EncrypedPost = collections.namedtuple(
    '_EncrypedPost',
    'encryption session symmetric_key encryption_url data identifier')


class _Post():
    """Abstract class to prepare data for posting to remote pattoo server."""
    def __init__(self, identifier, data):
        """Initialize the class.

        Args:
            identifier: Unique identifier for the source of the data. (AgentID)
            data: dict of data to post

        Returns:
            None

        """
        # Initialize key variables
        self.config = Config()

        # Get data and identifier
        self._data = data
        self._identifier = identifier

    def post(self):
        """Post data to API server.

        Args:
            None

        Returns:
            None

        """
        pass

    def purge(self):
        """Delete cached data and post to API server.

        Args:
            None

        Returns:
            None

        """
        pass


class Post(_Post):
    """Class to prepare data for posting to remote pattoo server."""

    def __init__(self, identifier, data):
        """Initialize the class.

        Args:
            identifier: Agent identifier
            data: Data from agent

        Returns:
            None

        """

        _Post.__init__(self, identifier, data)
        # URL to post to API server
        self._url = self.config.agent_api_server_url(identifier)

    def post(self):
        """Post data to central server.

        Args:
            None

        Returns:
            success: True: if successful

        """
        # Initialize key variables
        success = False

        # Post data
        if bool(self._data) is True:
            success = post(self._url, self._data, self._identifier)
        else:
            log_message = ('''\
Blank data. No data to post from identifier {}.'''.format(self._identifier))
            log.log2warning(1018, log_message)

        return success

    def purge(self):
        """Purge data from cache by posting to central server.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        purge(self._url, self._identifier)


class EncryptedPost(_Post):
    """Encrypted Post.

    Class to exchange public keys, set symmetric key and
    post symmetrically encrypted data to the API server

    First, the agent's information is exchanged. That
    information consists of the agent's email address
    and public key in ASCII. That information is received
    the the API server, the agent's public key is added to
    the API server's keyring and the agent's email address
    is stored in the API server's session to be used to
    retrieve the public key later on. Cookies are used to
    uniquely identity the agents. Secondly, the API
    server then sends a nonce encrypted by the agent's
    public key, the API sever's email address, and the
    API server's public key in ASCII. Then, the agent
    decrypts the nonce using its own private key. Having
    the decrypted nonce, the agent generates a symmetric
    key to symmetrically encrypt the nonce. The
    symmetric key is then encrypted using the API server's
    public key. Those two information are sent off to the
    API server. Finally, the encrypted symmetric key is
    decrypted using the API server's private key, then the
    symmetric key is used to decrypt the nonce. Once the
    nonce is verified to be the same that was sent off,
    the symmetric is stored, and all other information the
    API received is deleted. A response is sent to the
    agent and the agent proceeds to send data encrypted by
    the symmetric key. The data is decrypted once received
    by the API server. See encrypt.py for more details on
    the module.
    """

    def __init__(self, identifier, data, encryption):
        """Initialize the class.

        Args:
            identifier: Agent identifier
            data: Data from agent
            encryption: encrypt.Encryption object

        Returns:
            None

        """
        # Instantiate Post class
        _Post.__init__(self, identifier, data)

        # Get URLs for encryption
        self._encryption = encryption

    def purge(self):
        """Purge.

        Purge data from cache by posting encrypted data
        to the API server.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        success = False
        key = encrypt.generate_key(20)

        # Create a session and post data
        with requests.Session() as session:
            # Turn off HTTP Persistent connection
            session.keep_alive = False

            # Exchange keys
            success = key_exchange(
                _KeyExchange(
                    encryption=self._encryption,
                    session=session,
                    key_exchange_url=self.config.agent_api_key_url(),
                    symmetric_key_url=self.config.agent_api_validation_url(),
                    symmetric_key=key
                )
            )

        # Purge data, encrypt and send to API
        if success is True:
            purge(
                self.config.agent_api_encrypted_url(),
                self._identifier,
                suite=EncryptionSuite(
                    encrypted_post=encrypted_post,
                    encryption=self._encryption,
                    symmetric_key=key,
                    session=session
                )
            )

    def post(self):
        """Send encrypted data to the API server.

        Args:
            None

        Returns:
            success: True if data was posted successfully

        """
        # Initialize key variables
        success = False
        key = encrypt.generate_key(20)

        # Create a session and post data
        with requests.Session() as session:
            # Turn off HTTP Persistent connection
            session.keep_alive = False

            # Exchange keys
            result = key_exchange(
                _KeyExchange(
                    encryption=self._encryption,
                    session=session,
                    key_exchange_url=self.config.agent_api_key_url(),
                    symmetric_key_url=self.config.agent_api_validation_url(),
                    symmetric_key=key
                )
            )
            # Return if exchange failed
            if result is False:
                return success

            # Post data
            if bool(self._data) is True:
                success = encrypted_post(
                    _EncrypedPost(
                        encryption=self._encryption,
                        session=session,
                        symmetric_key=key,
                        encryption_url=self.config.agent_api_encrypted_url(),
                        data=self._data,
                        identifier=self._identifier
                    )
                )

            else:
                log_message = '''\
Blank data. No data to post from identifier {}.'''.format(self._identifier)
                log.log2warning(1056, log_message)

        return success


class PostAgent(Post):
    """Class to post AgentPolledData to remote pattoo server."""

    def __init__(self, agentdata):
        """Initialize the class.

        Args:
            agentdata: AgentPolledData object

        Returns:
            None

        """
        # Get extracted data
        identifier = agentdata.agent_id
        _data = converter.agentdata_to_post(agentdata)
        data = converter.posting_data_points(_data)

        # Log message that ties the identifier to an agent_program
        _log(agentdata.agent_program, identifier)

        # Don't post if agent data is invalid
        if agentdata.valid is False:
            data = None

        # Initialize key variables
        Post.__init__(self, identifier, data)


class EncryptedPostAgent(EncryptedPost):
    """Encrypted Post Agent.

    Class to prepare data for posting encrypted
    data to remote pattoo server."""

    def __init__(self, agentdata, encryption):
        """Initialize the class.

        Args:
            agentdata: Agent data
            encryption: encrypt.Encryption object

        Returns:
            None

        """
        # Get extracted data
        identifier = agentdata.agent_id
        _data = converter.agentdata_to_post(agentdata)
        data = converter.posting_data_points(_data)

        # Log message that ties the identifier to an agent_program
        _log(agentdata.agent_program, identifier)

        # Don't post if agent data is invalid
        if agentdata.valid is False:
            data = None

        # Initialize key variables
        EncryptedPost.__init__(self, identifier, data, encryption)


class PassiveAgent():
    """Gets data from passive Pattoo Agents for relaying to pattoo API."""

    def __init__(self, agent_program, identifier, url):
        """Initialize the class.

        Args:
            agent_program: Agent program name
            identifier: Unique identifier for the source of the data. (AgentID)
            url: URL of content to be retrieved from passive Pattoo agent

        Returns:
            None

        """
        # Initialize key variables
        self._url = url
        self._identifier = identifier
        self._agent_program = agent_program

    def relay(self):
        """Forward data polled from remote pattoo passive agent.

        Args:
            None

        Returns:
            None

        """
        # Get data
        data = self.get()
        identifier = self._identifier

        # Post data
        if bool(data) is True:
            # Log message that ties the identifier to an agent_program
            _log(self._agent_program, identifier)

            # Post to remote server
            server = Post(identifier, data)
            success = server.post()

            # Purge cache if success is True
            if success is True:
                server.purge()

    def get(self):
        """Get JSON from remote URL.

        Args:
            None

        Returns:
            result: dict of JSON retrieved.

        """
        # Initialize key variables
        result = {}
        url = self._url

        # Get URL
        try:
            with urllib.request.urlopen(url) as u_handle:
                try:
                    result = json.loads(u_handle.read().decode())
                except:
                    (etype, evalue, etraceback) = sys.exc_info()
                    log_message = (
                        'Error reading JSON from URL {}: [{}, {}, {}]'
                        ''.format(url, etype, evalue, etraceback))
                    log.log2info(1008, log_message)
        except:
            # Most likely no connectivity or the TCP port is unavailable
            (etype, evalue, etraceback) = sys.exc_info()
            log_message = (
                'Error contacting URL {}: [{}, {}, {}]'
                ''.format(url, etype, evalue, etraceback))
            log.log2info(1186, log_message)

        # Return
        return result


def post(url, data, identifier, save=True):
    """Post data to central server.

    Args:
        url: URL to receive posted data
        identifier: Unique identifier for the source of the data. (AgentID)
        data: Data dict to post. If None, then uses self._post_data (
            Used for testing and cache purging)
        save: When True, save data to cache directory if posting fails

    Returns:
        success: True: if successful

    """
    # Initialize key variables
    success = False
    response = False

    # Fail if nothing to post
    if isinstance(data, dict) is False or bool(data) is False:
        return success

    # Post data save to cache if this fails
    try:
        result = requests.post(url, json=data)
        response = True
    except:
        _exception = sys.exc_info()
        log_message = ('Data posting failure')
        log.log2exception(1097, _exception, message=log_message)
        if save is True:
            # Save data to cache
            _save_data(data, identifier)
        else:
            # Proceed normally if there is a failure.
            # This will be logged later
            pass

    # Define success
    if response is True:
        if result.status_code == 200:
            success = True
        else:
            log_message = ('''\
HTTP {} error for identifier "{}" posted to server {}\
'''.format(result.status_code, identifier, url))
            log.log2warning(1017, log_message)
            # Save data to cache, remote webserver isn't
            # working properly
            _save_data(data, identifier)

    # Log message
    if success is True:
        log_message = ('''\
Data for identifier "{}" posted to server {}\
'''.format(identifier, url))
        log.log2debug(1027, log_message)
    else:
        log_message = ('''\
Data for identifier "{}" failed to post to server {}\
'''.format(identifier, url))
        log.log2warning(1028, log_message)

    # Return
    return success


def key_exchange(metadata):
    """Exchange point for API and Agent public keys.

    Process:
        1) Post agent's email address and ASCII public key to API server.
        2) API server responds with its email address, ASCII pubic
           key and a nonce encrypted by the agent's public key.
        3) The agent decrypts the nonce and re-encrypts it with a randomly
           generated symmetric key.
        4) The symmetric key is then encrypted by the API server's public key.
        5) The encrypted nonce and encrypted symmetric key are sent to the API
           server.
        6) The API server decrypts this data and checks if nonce that was sent
           is the same as the nonce decrypted.
        7) If both nonces match, the server sends an OK signal to the agent.
        8) The symmetric key is used to encrypt data sent to the API server.

    Args:
        metadata: _KeyExchange object where:
            encryption: encrypt.Encryption object
            session: Requests session object
            symmetric_key: Symmetric key
            symmetric_key_url: API URL for symmetric key validation
            key_exchange_url: URL for key exchanges with the API server
            identifier: Agent identifier

    Returns:
        success: True if successful

    """
    # Predefine failure response
    success = False

    # Send the public key
    status = _send_agent_public_key(
        metadata.session,
        metadata.encryption,
        metadata.key_exchange_url
    )
    if status is False:
        return success

    # Get get API server's public key
    data = _get_api_public_key(
        metadata.session,
        metadata.key_exchange_url
    )
    if bool(data) is False:
        return success

    # Exchange symmetric key
    success = _send_symmetric_key(
        metadata.session,
        metadata.encryption,
        metadata.symmetric_key_url,
        metadata.symmetric_key,
        data
    )
    return success


def _send_agent_public_key(session, encryption, exchange_url):
    """Send public key to the remote API server.

    Args:
        session: Request Session object
        encryption: Encryption object
        exchange_url: URL to use for key exchange

    Returns:
        success: True is successful

    """
    # Predefine failure response
    success = False
    status = None

    # Data for POST
    send_data = {
        'pattoo_agent_email': encryption.email,
        'pattoo_agent_key': encryption.pexport()
    }

    # Convert dict to str
    send_data = json.dumps(send_data)

    try:
        # Send over data
        response = session.post(exchange_url, json=send_data)
        status = response.status_code
    except:
        _exception = sys.exc_info()
        log_message = ('Key exchange failure')
        log.log2exception(1077, _exception, message=log_message)

    # Checks that sent data was accepted
    if status in [202, 208]:
        success = True
    else:
        log_message = (
            'Cannot send public key to API server. Status: {}'.format(status))
        log.log2info(1069, log_message)

    return success


def _get_api_public_key(session, exchange_url):
    """Use previously established session to get the API server's public key.

    Args:
        session: Request Session object
        exchange_url: URL to use for key exchange

    Returns:
        result: JSON key data from API server

    """
    # Predefine failure response
    result = None
    status = None

    # Get API information
    try:
        response = session.get(exchange_url)
        status = response.status_code
    except:
        _exception = sys.exc_info()
        log_message = ('Key exchange failure')
        log.log2exception(1106, _exception, message=log_message)

    # Checks that the API sent over information
    if status == 200:
        # Process API server response
        result = response.json()
    else:
        log_message = (
            'Cannot get public key from API server. Status: {}'.format(status))
        log.log2info(1057, log_message)

    return result


def _send_symmetric_key(
        session, encryption, url, symmetric_key, data):
    """Send symmetric_key to the remote API server.

    Args:
        session: Request Session object
        encryption: Encryption object
        url: URL to use for exchanging the symmetric key
        symmetric_key: Symmetric key
        data: Data to post

    Returns:
        success: True if successful

    """
    # Predefine failure response
    success = False
    status = None

    # Process API server information
    api_email = data['api_email']
    api_key = data['api_key']
    encrypted_nonce = data['encrypted_nonce']

    # Import API public key
    encryption.pimport(api_key)
    api_fingerprint = encryption.fingerprint(api_email)
    encryption.trust(api_fingerprint)

    # Decrypt nonce
    decrypted_nonce = encryption.decrypt(encrypted_nonce)

    # Create JSON to post
    data_ = json.dumps(
        {
            'encrypted_nonce': encryption.sencrypt(
                decrypted_nonce, symmetric_key),
            'encrypted_sym_key': encryption.encrypt(
                symmetric_key, api_fingerprint)
        }
    )

    # POST data to API
    try:
        response = session.post(url, json=data_)
        status = response.status_code
    except:
        _exception = sys.exc_info()
        log_message = ('Symmetric key exchange failure')
        log.log2exception(1098, _exception, message=log_message)

    # Check that the transaction was validated
    if status == 200:
        success = True
    else:
        log_message = '''\
Cannot exchange symmetric keys with API server. Status: {}'''.format(status)
        log.log2info(1099, log_message)

    return success


def encrypted_post(metadata, save=True):
    """Post encrypted data to the API server.

    First, the data is checked for its validity. Sencondly,
    the data and agent ID is stored in a dictionary with
    the key value pairs. The dictionary is converted to a
    string so that is can be encrypted. The encrypted data
    is then paired with a key, as a dictionary, distinguishing
    the data as encrypted. The dictionary is then converted
    to a string so it can be added to the request method
    as json. A response from the API server tells if the data
    was received and decrypted successfully.

    Args:
        metadata: _EncrypedPost object where:
            encryption: encrypt.Encryption object
            session: Requests session object
            symmetric_key: Symmetric key
            encryption_url: API URL to post the data to
            data: Data to post as a dict
            identifier: Agent identifier
        save: If True, save data to cache if API server is inaccessible

    Returns:
        success: True if successful

    """
    # Initialize key variables
    success = False
    status = None

    # Fail if nothing to post
    if isinstance(metadata.data, dict) is False or bool(
            metadata.data) is False:
        return success

    # Prepare data for posting
    data = json.dumps(
        {
            'data': metadata.data,
            'source': metadata.identifier
        }
    )

    # Symmetrically encrypt data
    encrypted_data = metadata.encryption.sencrypt(data, metadata.symmetric_key)

    # Post data save to cache if this fails
    try:
        response = metadata.session.post(
            metadata.encryption_url,
            json=json.dumps({'encrypted_data': encrypted_data})
        )
        status = response.status_code
    except:
        _exception = sys.exc_info()
        log_message = ('Encrypted posting failure')
        log.log2exception(1075, _exception, message=log_message)
        if save is True:
            # Save data to cache
            _save_data(metadata.data, metadata.identifier)
        else:
            # Proceed normally if there is a failure.
            # This will be logged later
            pass

    # Checks if data was posted successfully
    if status == 202:
        log_message = 'Posted to API. Response "{}" from URL: "{}"'.format(
            status, metadata.encryption_url)
        log.log2debug(1059, log_message)

        # The data was accepted successfully
        success = True
    else:
        log_message = 'Error posting. Response "{}" from URL: "{}"'.format(
            status, metadata.encryption_url)
        log.log2warning(1058, log_message)

    return success


def purge(url, identifier, suite=post):
    """Purge data from cache by posting to central server.

    Args:
        url: URL to receive posted data
        identifier: Unique identifier for the source of the data. (AgentID)
        suite: If a function, this will proceed to use the normal post function
            for unencrypted posting. If EncryptionSuite, the necessary
            variables from the named tuple will be used along with the
            encrypted_post function for encrypted posting

    Returns:
        None

    """
    # Initialize key variables
    config = Config()
    cache_dir = config.agent_cache_directory(identifier)

    # Add files in cache directory to list only if they match the
    # cache suffix
    all_filenames = [filename for filename in os.listdir(
        cache_dir) if os.path.isfile(
            os.path.join(cache_dir, filename))]
    filenames = [
        filename for filename in all_filenames if filename.endswith(
            '.json')]

    # Read cache file
    for filename in filenames:
        # Only post files for our own UID value
        if identifier not in filename:
            continue

        # Get the full filepath for the cache file and post
        filepath = os.path.join(cache_dir, filename)
        with open(filepath, 'r') as f_handle:
            try:
                data = json.load(f_handle)
            except:
                # Log removal
                log_message = ('''\
Error reading previously cached agent data file {} for identifier {}. May be \
corrupted.'''.format(filepath, identifier))
                log.log2warning(1064, log_message)

                # Delete file
                if os.path.isfile(filepath) is True:
                    os.remove(filepath)

                    log_message = ('''\
Deleting corrupted cache file {} for identifier {}.\
'''.format(filepath, identifier))
                    log.log2warning(1036, log_message)

                # Go to the next file.
                continue

        # Post file
        if callable(suite):  # Is it a function?
            # Post unencrypted data
            success = suite(url, data, identifier, save=False)
        elif isinstance(suite, EncryptionSuite):  # Is it EncryptionSuite?
            # Post encrypted data
            success = suite.encrypted_post(
                _EncrypedPost(
                    encryption=suite.encryption,
                    session=suite.session,
                    symmetric_key=suite.symmetric_key,
                    encryption_url=url,
                    data=data,
                    identifier=identifier
                )
            )

        # Delete file if successful
        if success is True:
            if os.path.exists(filepath) is True:
                os.remove(filepath)

                # Log removal
                log_message = ('''\
    Purging cache file {} after successfully contacting server {}\
    '''.format(filepath, url))
                log.log2info(1007, log_message)


def _save_data(data, identifier):
    """Save data to cache file.

    Args:
        data: Dict to save
        identifier: Unique identifier for the source of the data. (AgentID)

    Returns:
        success: True: if successful

    """
    # Initialize key variables
    success = False
    config = Config()
    cache_dir = config.agent_cache_directory(identifier)
    timestamp = int(time() * 1000)

    # Create a unique very long filename to reduce risk of
    filename = ('''{}{}{}_{}.json\
'''.format(cache_dir, os.sep, timestamp, identifier))

    # Save data
    try:
        with open(filename, 'w') as f_handle:
            json.dump(data, f_handle)
        success = True
    except Exception as err:
        log_message = '{}'.format(err)
        log.log2warning(1030, log_message)
    except:
        (etype, evalue, etraceback) = sys.exc_info()
        log_message = ('''\
Cache-file save error: [{}, {}, {}]'''.format(etype, evalue, etraceback))
        log.log2warning(1031, log_message)

    # Delete file if there is a failure.
    # Helps to protect against full file systems.
    if os.path.isfile(filename) is True and success is False:
        os.remove(filename)
        log_message = ('''\
Deleting corrupted cache file {} for identifier {}.\
'''.format(filename, identifier))
        log.log2warning(1037, log_message)

    # Return
    return success


def _log(agent_program, identifier):
    """Create a standardized log message for posting.

    Args:
        agent_program: Agent program name
        identifier: Unique identifier for the source of the data. (AgentID)

    Returns:
        None

    """
    # Log message that ties the identifier to an agent_program
    log_message = ('''\
Agent program {} posting data as {}'''.format(agent_program, identifier))
    log.log2debug(1038, log_message)



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import data
from pattoo_shared.constants import (
    DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT64)
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_hashstring(self):
        """Testing function hashstring."""
        # Test
        value = 'unittest'

        # sha256
        result = data.hashstring(value)
        expected = '''\
df08227309c92f2f3f030b423009e697445371283d89352a26d064d39fb73c36'''
        self.assertEqual(result, expected)
        result = data.hashstring(value, sha=256)
        self.assertEqual(result, expected)

        # sha1
        result = data.hashstring(value, sha=1)
        expected = '''\
94e060874450b5ea724bb6ce5ca7be4f6a73416b'''
        self.assertEqual(result, expected)

        # sha1 UTF8
        result = data.hashstring(value, sha=1, utf8=True)
        expected = b'94e060874450b5ea724bb6ce5ca7be4f6a73416b'
        self.assertEqual(result, expected)

        # sha384
        result = data.hashstring(value, sha=384)
        expected = '''\
99ec86e99be2d6a9d9ad4947cb7a3aa43df85667a5a2839aeb3cedf0baa132a852381f94c70bb\
a8576b7df11a2d3b819'''
        self.assertEqual(result, expected)

        # sha512
        result = data.hashstring(value, sha=512)
        expected = '''\
def51cb07699f90b1613d0f4da13574e415323b2fcb98c1f072218a8ba82a44432da79bdf90ebf\
82581d933dc4128e83bda0c9f9f7b8b32e41b8ee8bb16be531'''
        self.assertEqual(result, expected)

    def test_is_numeric(self):
        """Testing function is_numeric."""
        # Test False
        items = [None, True, False, 'False']
        for item in items:
            result = data.is_numeric(item)
            self.assertFalse(result)

        # Test True
        items = [1, 1.1, '1.1']
        for item in items:
            result = data.is_numeric(item)
            self.assertTrue(result)

    def test_is_data_type_numeric(self):
        """Testing function is_data_type_numeric."""
        # Test False
        items = [None, True, False, 'False', 20, 30, 40]
        for item in items:
            result = data.is_data_type_numeric(item)
            self.assertFalse(result)

        # Test True
        items = [
            99, 101, 32, 64, DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT64]
        for item in items:
            result = data.is_data_type_numeric(item)
            self.assertTrue(result)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo Data Converter."""

# Standard imports
import re

# Pattoo libraries
from .variables import (
    ConverterMetadata, DataPoint, AgentPolledData,
    PostingDataPoints)
from .constants import (
    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE,
    MAX_KEYPAIR_LENGTH, PattooDBrecord, RESERVED_KEYS, CACHE_KEYS,
    AGENT_METADATA_KEYS)
from pattoo_shared import data
from pattoo_shared import log


class Counter():
    """Count and format datapoint key-value pairs."""

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        Variables:
            self._count: Number of key-value pairs processed

        """
        # Initialize key variables
        self._count = 0
        self.pairs = {}
        self.inverse_pairs = {}

    def counter(self, key, value):
        """Initialize the class.

        Args:
            key: Key
            value: Value

        Returns:
            result: Tuple of ((key, value), count)

        """
        # Return
        pair = (key, value)
        if pair not in self.pairs:
            self.pairs[pair] = self._count
            self.inverse_pairs[self._count] = pair
            self._count += 1
        result = self.pairs[pair]
        return result


def cache_to_keypairs(_data):
    """Convert agent cache data to AgentPolledData object.

    Args:
        _data: Data read from JSON cache file

    Returns:
        result: Validated cache data. [] if invalid.

    """
    # Initialize key variables
    result = []
    _log_message = 'Invalid cache data.'

    # Basic validation
    if isinstance(_data, dict) is False:
        log.log2warning(1032, _log_message)
        return []
    if len(_data) != len(CACHE_KEYS):
        log.log2warning(1033, _log_message)
        return []
    for key in _data.keys():
        if key not in CACHE_KEYS:
            log.log2warning(1034, _log_message)
            return []

    ####################################################################
    # Verify pattoo_datapoints
    ####################################################################

    # Verify we are getting a dict of datapoints
    if isinstance(_data['pattoo_datapoints'], dict) is False:
        log.log2warning(1035, _log_message)
        return []

    # Verify we are getting the correct key count
    if len(_data['pattoo_datapoints']) != 2:
        log.log2warning(1048, _log_message)
        return []

    # Verify we are getting the correct keys
    for item in ['key_value_pairs', 'datapoint_pairs']:
        if item not in _data['pattoo_datapoints']:
            log.log2warning(1049, _log_message)
            return []

    # Verify there are datapoint defining keys
    if isinstance(
            _data['pattoo_datapoints']['key_value_pairs'], dict) is False:
        log.log2warning(1050, _log_message)
        return []
    if isinstance(
            _data['pattoo_datapoints']['datapoint_pairs'], list) is False:
        log.log2warning(1051, _log_message)
        return []

    # Prepare for datapoint processing
    key_value_pairs = _data['pattoo_datapoints']['key_value_pairs']
    datapoint_pairs = _data['pattoo_datapoints']['datapoint_pairs']

    # Process each datapoint
    for pair_ids in datapoint_pairs:
        item = {}

        # Validate and assign key-values from datapoints
        for pair_id in pair_ids:
            # Lookup on a string of pair_id as the JSON in the cache file is
            # keyed by string integers
            _kv = key_value_pairs.get(str(pair_id))
            if isinstance(_kv, list) is False:
                log.log2warning(1046, _log_message)
                return []
            if len(_kv) != 2:
                log.log2warning(1045, _log_message)
                return []
            (key, value) = _kv
            item[key] = value

        # Assign datapoint values to PattooDBrecord
        pattoo_db_variable = _make_pattoo_db_record(item)
        if bool(pattoo_db_variable) is True:
            result.append(pattoo_db_variable)

    # Return
    return result


def _make_pattoo_db_record(item):
    """Ingest data.

    Args:
        item: Dict of key-value pairs DataPoint

    Returns:
        pattoo_db_variable: PattooDBrecord object

    """
    # Initialize data
    valids = []
    pattoo_db_variable = None
    _log_message = 'Invalid cache data.'
    reserved_keys_non_metadata = [
        _ for _ in RESERVED_KEYS if _ != 'pattoo_metadata']
    metadata = {}

    '''
    Make sure we have all keys required for creating a PattooDBrecord
    Omit 'pattoo_metadata' as we need to recreate it. 'pattoo_metadata' was
    extracted to its component key-value pairs before the agent posted it to
    the pattoo API
    '''
    for key in reserved_keys_non_metadata:
        valids.append(key in item.keys())
    for key in AGENT_METADATA_KEYS:
        valids.append(key in item.keys())
    if False in valids:
        log.log2warning(1047, _log_message)
        return None

    # Get metadata for item
    for key, value in sorted(item.items()):
        if key not in reserved_keys_non_metadata:
            metadata[key] = value
            valids.append(isinstance(key, str))

        # Work on the data_type
        if key == 'pattoo_data_type':
            valids.append(value in [
                DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT,
                DATA_STRING, DATA_NONE])

    # Append to result
    if False not in valids:
        # Add the datasource to the original checksum for better uniqueness
        checksum = _checksum(
            item['pattoo_agent_id'],
            item['pattoo_agent_polled_target'],
            item['pattoo_checksum'])
        pattoo_db_variable = PattooDBrecord(
            pattoo_checksum=checksum,
            pattoo_key=item['pattoo_key'],
            pattoo_agent_id=item['pattoo_agent_id'],
            pattoo_agent_polling_interval=item[
                'pattoo_agent_polling_interval'],
            pattoo_timestamp=item['pattoo_timestamp'],
            pattoo_data_type=item['pattoo_data_type'],
            pattoo_value=item['pattoo_value'],
            pattoo_agent_polled_target=item['pattoo_agent_polled_target'],
            pattoo_agent_program=item['pattoo_agent_program'],
            pattoo_agent_hostname=item['pattoo_agent_hostname'],
            pattoo_metadata=_keypairs(metadata)
        )

    # Return
    return pattoo_db_variable


def agentdata_to_datapoints(agentdata):
    """Ingest data.

    Args:
        agentdata: AgentPolledData object

    Returns:
        rows: List of DataPoint objects

    """
    # Initialize key variables
    rows = []

    # Only process valid data
    if isinstance(agentdata, AgentPolledData) is True:
        # Return if invalid data
        if bool(agentdata.valid) is False:
            return []

        for ddv in agentdata.data:
            # Ignore bad data
            if ddv.valid is False:
                continue

            # Get data
            for _dv in ddv.data:
                # Assign values to DataPoints
                metadata = {
                    True: {
                        'pattoo_agent_id': agentdata.agent_id,
                        'pattoo_agent_program': agentdata.agent_program,
                        'pattoo_agent_hostname': agentdata.agent_hostname,
                        'pattoo_agent_polled_target': ddv.target,
                    },
                    False: {
                        'pattoo_agent_polling_interval': (
                            agentdata.agent_polling_interval)
                        }
                }
                for update_checksum, items in sorted(metadata.items()):
                    for key, value in sorted(items.items()):
                        _dv.add(ConverterMetadata(
                            key, value, update_checksum=update_checksum))
                rows.append(_dv)

    # Return
    return rows


def datapoints_to_dicts(items):
    """Convert a list of DataPoint objects to a standardized dict for posting.

    Args:
        items: List of datapoints to convert

    Returns:
        result: Dict of 'key_value_pairs' and 'pattoo_datapoints' dicts
            key_value_pairs = Keyed by (key, value) with an ID value
            pattoo_datapoints = List of ID values with unique datapoint ID key

    """
    # Initialize key variables
    datapoints = []
    counter = Counter()
    all_dps = []

    # Verify input data
    if isinstance(items, list) is False:
        items = [items]
    for item in items:
        if isinstance(item, DataPoint):
            datapoints.append(item)

    # Populate dict to get unique key-value pairs
    for datapoint in datapoints:
        # Only convert valid data
        if datapoint.valid is True:
            dp_pair_ids = []
            dp_key_values = {}

            # Convert metadata into list of tuples
            key_values = list(datapoint.metadata.items())

            # Convert non metadata into list of tuples
            key_values.extend([
                ('pattoo_key', datapoint.key),
                ('pattoo_data_type', datapoint.data_type),
                ('pattoo_value', datapoint.value),
                ('pattoo_timestamp', datapoint.timestamp),
                ('pattoo_checksum', datapoint.checksum)])

            # Process tuples
            for key, value in key_values:
                # Assign ID to each key-value pair and store for later
                id_pair = counter.counter(key, value)
                dp_pair_ids.append(id_pair)
                dp_key_values[key] = value

            # Create a unique key tuple for the datapoint
            all_dps.append(dp_pair_ids)

    result = {
        'key_value_pairs': counter.inverse_pairs,
        'datapoint_pairs': all_dps}
    return result


def agentdata_to_post(agentdata):
    """Create data to post to the pattoo API.

    Args:
        agentdata: AgentPolledData object

    Returns:
        result: Dict of data to post

    """
    # Initialize key Variables
    agent_id = agentdata.agent_id
    polling_interval = agentdata.agent_polling_interval
    _data = agentdata_to_datapoints(agentdata)
    _datapoints = datapoints_to_dicts(_data)
    result = datapoints_to_post(agent_id, polling_interval, _datapoints)
    return result


def datapoints_to_post(agent_id, polling_interval, datapoints):
    """Create data to post to the pattoo API.

    Args:
        agent_id: Unique ID of agent posting data
        polling_interval: Interval over which the data is periodically polled
        datapoints: List of DataPoint objects

    Returns:
        result: Dict of data to post

    """
    result = PostingDataPoints(agent_id, polling_interval, datapoints)
    return result


def posting_data_points(_data):
    """Create data to post to the pattoo API.

    Args:
        _data: PostingDataPoints object

    Returns:
        result: Dict of data to post

    """
    result = {
        'pattoo_agent_timestamp': _data.pattoo_timestamp,
        'pattoo_agent_id': _data.pattoo_agent_id,
        'pattoo_agent_polling_interval': _data.pattoo_agent_polling_interval,
        'pattoo_datapoints': _data.pattoo_datapoints}
    return result


def _keypairs(_data):
    """Make key-pairs from metadata dict.

    Args:
        data: Metadata dict

    Returns:
        result: List of tuples of key-pair values

    """
    # Initialize key variables
    result = []

    # Loop around keys
    for _key, value in _data.items():
        # We want to make sure that we don't have
        # duplicate key-value pairs
        if _key in RESERVED_KEYS:
            continue
        # Key-Value pairs must be strings
        if isinstance(_key, str) is False or isinstance(
                value, str) is False:
            continue

        # Standardize the keys use underscores to separate words
        splits = re.findall(r"[\w']+", _key)
        key = '_'.join(splits).lower()

        # Update the list
        result.append(
            (str(key)[:MAX_KEYPAIR_LENGTH], str(
                value)[:MAX_KEYPAIR_LENGTH])
        )

    return result


def _checksum(agent_id, target, datapoint_checksum):
    """Create a unique checksum for a DataPoint based on agent and target.

    Args:
        record: PattooDBrecord converted to a Dict
        metadat: DataPoint Metadata

    Returns:
        result: Checksum

    """
    # Create checksum value
    result = data.hashstring('''{}{}{}\
'''.format(agent_id, target, datapoint_checksum), sha=512)
    return result



#!/usr/bin/env python3
"""Test pattoo shared shared script."""

import os
import sys
import unittest

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(
        os.path.abspath(os.path.join(
            EXEC_DIR, os.pardir)), os.pardir)), os.pardir))
_EXPECTED = '''\
{0}pattoo-shared{0}tests{0}pattoo_shared_{0}installation'''.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from tests.libraries.configuration import UnittestConfig
from pattoo_shared.installation import shared


class Test_Shared(unittest.TestCase):
    """Checks all functions for the Pattoo config script."""

    def test_log(self):
        """Unittest to test the _log function."""
        with self.assertRaises(SystemExit) as cm_:
            shared.log("Test Error Message")
        self.assertEqual(cm_.exception.code, 3)

    def test_run_script(self):
        """Unittest to test the _run_script function."""
        # Test case where the script should fail and exit with 2
        with self.subTest():
            with self.assertRaises(SystemExit) as cm_:
                shared.run_script('this will exit with 2')
            self.assertEqual(cm_.exception.code, 2)

        # Test case where the script should print "this works" to the console
        with self.subTest():
            expected = 0
            result = shared.run_script('echo hello world')[0]
            self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



"""Module of general functions."""

# Standard imports
import random
import hashlib

# Import from the PattooShared library
from pattoo_shared.constants import DATA_FLOAT
from pattoo_shared.variables import (
    DataPoint, DataPointMetadata, TargetDataPoints, AgentPolledData)


def random_string():
    """Creates random string.

    Args:
        None

    Return:
        result: string

    """
    # Return
    result = random_agent_name()
    return result


def random_agent_name():
    """Creates random agent name.

    Args:
        None

    Return:
        result: Agent name

    """
    salt = str(random.getrandbits(128))
    result = hashlib.sha224(salt.encode()).hexdigest()
    return result


def test_agent():
    # Define the polling interval in seconds (integer).
    polling_interval = 300

    # Give the agent a name
    agent_name = 'sample_agent_script'

    # Let's assume the script has already received this data from SITE_A
    site_a_data = [
        ['ABC', 123.456],
        ['DEF', 456.789]
    ]

    # Let's assume the script has already received this data from WORK_1
    work_1_data = [
        ['GHI', 654.321],
        ['JKL', 987.654]
    ]

    # Setup the agent's AgentPolledData object.
    agent = AgentPolledData(agent_name, polling_interval)

    # Let's add some metadata that you don't want to affect charting in the
    # event of a change. Department names change all the time.
    metadata_static = DataPointMetadata(
        'Department Name', 'The Palisadoes Foundation', update_checksum=False)

    # Let's add some metadata that will change and trigger a new chart.
    metadata_dynamic = DataPointMetadata('Financial Year', '2020')

    # Create target objects for SITE_A
    target = TargetDataPoints('SITE_A')
    for quote in site_a_data:
        key, value = quote
        datapoint = DataPoint(key, value, data_type=DATA_FLOAT)
        datapoint.add(metadata_static)
        datapoint.add(metadata_dynamic)
        target.add(datapoint)
    agent.add(target)

    # Create target objects for WORK_1
    target = TargetDataPoints('WORK_1')
    for quote in work_1_data:
        key, value = quote
        datapoint = DataPoint(key, value, data_type=DATA_FLOAT)
        datapoint.add(metadata_static)
        datapoint.add(metadata_dynamic)
        target.add(datapoint)
    agent.add(target)

    # Return agent
    return agent



"""Module that defines constants shared between pattoo and its agents.

The aim is to have a single location for constants that may be used across
agents to prevent the risk of duplication.

"""
import collections

###############################################################################
# Universal constants for all agents
###############################################################################

DATA_FLOAT = 101
DATA_INT = 99
DATA_COUNT64 = 64
DATA_COUNT = 32
DATA_STRING = 2
DATA_NONE = None

###############################################################################
# Constants for data DB ingestion
###############################################################################

MAX_KEYPAIR_LENGTH = 512

# Groupings of reserved keys
DATAPOINT_KEYS = (
    'pattoo_checksum', 'pattoo_metadata', 'pattoo_data_type', 'pattoo_key',
    'pattoo_value', 'pattoo_timestamp'
)

# Metadata keys added to datapoints by agents
AGENT_METADATA_KEYS = (
    'pattoo_agent_id', 'pattoo_agent_polled_target', 'pattoo_agent_program',
    'pattoo_agent_hostname', 'pattoo_agent_polling_interval')

# Create reserved keys
_ = list(DATAPOINT_KEYS)
_.extend(AGENT_METADATA_KEYS)
RESERVED_KEYS = tuple(_)

PattooDBrecord = collections.namedtuple(
    'PattooDBrecord', ' '.join(RESERVED_KEYS))

# Keys of posted cached data. Based on keys in
# pattoo_shared.constants.PostingDataPoints
CACHE_KEYS = (
    'pattoo_agent_id', 'pattoo_datapoints', 'pattoo_agent_polling_interval',
    'pattoo_agent_timestamp')

###############################################################################
# Constants for pattoo Agent API
###############################################################################

_PATTOO = '/pattoo'
PATTOO_WEB_SITE_PREFIX = '{}'.format(_PATTOO)
PATTOO_API_SITE_PREFIX = '{}/api/v1'.format(_PATTOO)
PATTOO_API_AGENT_PREFIX = '{}/agent'.format(PATTOO_API_SITE_PREFIX)
PATTOO_API_WEB_PREFIX = '{}/web'.format(PATTOO_API_SITE_PREFIX)



#!/usr/bin/env python3
"""Class used to create the configuration file used for unittesting.

NOTE!! This script CANNOT import any pattoo-shared libraries. Doing so risks
libraries trying to access a configuration or configuration directory that
doesn't yet exist. This is especially important when running cloud based
automated tests such as 'Travis CI'.

"""

# Standard imports
from __future__ import print_function
import tempfile
import os
import yaml

PATTOO_API_AGENT_NAME = 'pattoo_api_agentd'

# Pattoo imports
from pattoo_shared import log
from pattoo_shared import configuration


class TestConfigAgentAPId(configuration.ServerConfig):
    """Class gathers all configuration information.

    Only processes the following YAML keys in the configuration file:

        The value of the PATTOO_API_WEB_NAME constant

    """

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        """
        # Instantiate the Config parent
        configuration.ServerConfig.__init__(self)

    def ip_listen_address(self):
        """Get ip_listen_address.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        key = PATTOO_API_AGENT_NAME
        sub_key = 'ip_listen_address'
        result = configuration.search(
            key, sub_key, self._server_yaml_configuration, die=False)

        # Default to 0.0.0.0
        if result is None:
            result = '0.0.0.0'
        return result

    def ip_bind_port(self):
        """Get ip_bind_port.

        Args:
            None

        Returns:
            result: result

        """
        # Initialize key variables
        key = PATTOO_API_AGENT_NAME
        sub_key = 'ip_bind_port'

        # Get result
        intermediate = configuration.search(
            key, sub_key, self._server_yaml_configuration, die=False)
        if intermediate is None:
            result = 20202
        else:
            result = int(intermediate)
        return result

    def api_email_address(self):
        """GET API email address from yaml file.

        Args:
            None

        Returns:
            email (str): Email address of API
        """
        # Initialize key variables
        key = PATTOO_API_AGENT_NAME
        sub_key = 'api_encryption_email'

        result = configuration.search(
            key, sub_key, self._server_yaml_configuration, die=False)
        if result is None:
            result = 'pattoo_api@example.org'
        return result


class UnittestConfig():
    """Creates configuration for testing."""

    def __init__(self):
        """Initialize the class."""
        # Initialize GLOBAL variables
        config_suffix = '.pattoo-shared-unittests{0}config'.format(os.sep)
        self._config_directory = (
            '{}{}{}'.format(os.environ['HOME'], os.sep, config_suffix))

        # Make sure the environmental variables are OK
        _environment(self._config_directory)

        # Set global variables
        self._log_directory = tempfile.mkdtemp()
        self._cache_directory = tempfile.mkdtemp()
        self._daemon_directory = tempfile.mkdtemp()
        self._system_daemon_directory = tempfile.mkdtemp()

        # Make sure the configuration directory is OK
        if os.path.isdir(self._config_directory) is False:
            os.makedirs(self._config_directory, mode=0o750, exist_ok=True)

        self._config = {
            'pattoo': {
                'log_directory': self._log_directory,
                'log_level': 'debug',
                'language': 'xyz',
                'cache_directory': self._cache_directory,
                'daemon_directory': self._daemon_directory,
                'system_daemon_directory': self._system_daemon_directory,
            },
            'encryption': {
                'api_email': 'test_api@example.org',
            }
        }
        self._agent_config = {
            'pattoo_agent_api': {
                'ip_address': '127.0.0.6',
                'ip_bind_port': 50505,
            },

            'encryption': {
                'agent_email': 'test_agent@example.org'
            }
        }
        self._server_config = {
            'pattoo_api_agentd': {
                'ip_listen_address': '0.0.0.0',
                'ip_bind_port': 60606,
                'api_encryption_email': 'test_api@example.org'
            },
        }

    def create(self):
        """Create a good config and set the PATTOO_CONFIGDIR variable.

        Args:
            None

        Returns:
            self.config_directory: Directory where the config is placed

        """
        # Initialize key variables
        filenames = {
            '{}{}pattoo.yaml'.format(
                self._config_directory, os.sep): self._config,
            '{}{}pattoo_agent.yaml'.format(
                self._config_directory, os.sep): self._agent_config,
            '{}{}pattoo_server.yaml'.format(
                self._config_directory, os.sep): self._server_config
        }

        for filename, content in filenames.items():
            # Write to pattoo.yaml
            try:
                fh_ = open(filename, 'w')
            except PermissionError:
                log.log2die(1019, '''\
    Insufficient permissions for creating the file: {}'''.format(filename))
            else:
                with fh_:
                    # Convert to dict, just in case as defaultdict
                    # isn't supported by yaml.safe_dump
                    yaml.safe_dump(
                        dict(content), stream=fh_, default_flow_style=False)

        # Return
        return self._config_directory

    def cleanup(self):
        """Remove all residual directories.

        Args:
            None

        Returns:
            None

        """
        # Delete directories
        directories = [
            self._log_directory,
            self._cache_directory,
            self._daemon_directory,
            self._config_directory]
        for directory in directories:
            _delete_files(directory)


def _delete_files(directory):
    """Delete all files in directory."""
    # Cleanup files in temp directories
    filenames = [filename for filename in os.listdir(
        directory) if os.path.isfile(
            os.path.join(directory, filename))]

    # Get the full filepath for the cache file and remove filepath
    for filename in filenames:
        filepath = os.path.join(directory, filename)
        os.remove(filepath)

    # Remove directory after files are deleted.
    os.rmdir(directory)


def _environment(config_directory):
    """Make sure environmental variables are OK.

    Args:
        config_directory: Directory with the configuration

    Returns:
        None

    """
    # Create a message for the screen
    screen_message = ('''
The PATTOO_CONFIGDIR is set to the wrong directory. Run this command to do \
so:

$ export PATTOO_CONFIGDIR={}

Then run this command again.
'''.format(config_directory))

    # Make sure the PATTOO_CONFIGDIR environment variable is set
    if 'PATTOO_CONFIGDIR' not in os.environ:
        log.log2die_safe(1023, screen_message)

    # Make sure the PATTOO_CONFIGDIR environment variable is set correctly
    if os.environ['PATTOO_CONFIGDIR'] != config_directory:
        log.log2die_safe(1024, screen_message)

    # Update message
    screen_message = ('''{}

PATTOO_CONFIGDIR is incorrectly set to {}

'''.format(screen_message, os.environ['PATTOO_CONFIGDIR']))

    # Make sure the PATTOO_CONFIGDIR environment variable is set to unittest
    if 'unittest' not in os.environ['PATTOO_CONFIGDIR']:
        log_message = (
            'The PATTOO_CONFIGDIR is not set to a unittest directory')
        log.log2die_safe(1025, log_message)



#!/usr/bin/env python3
"""Nagios check logging library."""

from __future__ import print_function
import sys
import os
import datetime
import time
import getpass
import logging
import traceback

# Define global variable
LOGGER = {}


def check_environment():
    """Check environmental variables. Die if incorrect.

    Args:
        None

    Returns:
        path: Path to the configurtion directory

    """
    # Get environment
    if 'PATTOO_CONFIGDIR' not in os.environ:
        os.environ['PATTOO_CONFIGDIR'] = '/etc/pattoo'

    # Verify configuration directory
    config_directory = os.environ['PATTOO_CONFIGDIR']
    if (os.path.exists(config_directory) is False) or (
            os.path.isdir(config_directory) is False):
        log_message = (
            'Environment variable $PATTOO_CONFIGDIR set to '
            'directory {} that does not exist'
            ''.format(config_directory))
        # Must print statement as logging requires a config directory
        log2die_safe(1020, log_message)

    # Return
    path = os.environ['PATTOO_CONFIGDIR']
    return path


class _GetLog():
    """Class to manage the logging without duplicates."""

    def __init__(self):
        """Initialize the class."""
        # Pattoo libraries
        from pattoo_shared.configuration import BaseConfig

        # Define key variables
        app_name = 'pattoo'
        levels = {
            'debug': logging.DEBUG,
            'info': logging.INFO,
            'warning': logging.WARNING,
            'error': logging.ERROR,
            'critical': logging.CRITICAL
        }

        # Get the logging directory
        config = BaseConfig()
        log_file = config.log_file()
        config_log_level = config.log_level()

        # Set logging level
        if config_log_level in levels:
            log_level = levels[config_log_level]
        else:
            log_level = levels['debug']

        # create logger with app_name
        self.logger_file = logging.getLogger('{}_file'.format(app_name))
        self.logger_stdout = logging.getLogger('{}_console'.format(app_name))

        # Set logging levels to file and stdout
        self.logger_stdout.setLevel(log_level)
        self.logger_file.setLevel(log_level)

        # create file handler which logs even debug messages
        file_handler = logging.FileHandler(log_file)
        file_handler.setLevel(log_level)

        # create console handler with a higher log level
        stdout_handler = logging.StreamHandler()
        stdout_handler.setLevel(log_level)

        # create formatter and add it to the handlers
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        file_handler.setFormatter(formatter)
        stdout_handler.setFormatter(formatter)

        # add the handlers to the logger
        self.logger_file.addHandler(file_handler)
        self.logger_stdout.addHandler(stdout_handler)

    def logfile(self):
        """Return logger for file IO.

        Args:
            None

        Returns:
            value: Value of logger

        """
        # Return
        value = self.logger_file
        return value

    def stdout(self):
        """Return logger for terminal IO.

        Args:
            None

        Returns:
            value: Value of logger

        """
        # Return
        value = self.logger_stdout
        return value


def log2console(code, message):
    """Log message to STDOUT only and die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Initialize key variables
    output = _message(code, message, False)
    print(output)


def log2die_safe_exception(code, sys_exc_info, message):
    """Log trace message to file and STDOUT, but don't die.

    Args:
        code: Message code
        sys_exc_info: Tuple from exception from sys.exc_info
        message: Message text

    Returns:
        None

    """
    # Initialize key variables
    (exc_type, exc_value, exc_traceback) = sys_exc_info
    log_message = ('''\
[Bug: Exception Type:{}, Exception Instance: {}, Stack Trace Object: {}] {}\
'''.format(exc_type, exc_value, exc_traceback, message))
    log2die_safe(code, log_message)


def log2die_safe(code, message):
    """Log message to STDOUT only and die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Initialize key variables
    output = _message(code, message, True)
    print(output)
    sys.exit(2)


def log2warning(code, message):
    """Log warning message to file only, but don't die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Initialize key variables
    _logit(code, message, error=False, verbose=False, level='warning')


def log2debug(code, message):
    """Log debug message to file only, but don't die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Initialize key variables
    _logit(code, message, error=False, verbose=False, level='debug')


def log2info(code, message):
    """Log status message to file only, but don't die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Log to screen and file
    _logit(code, message, error=False, verbose=False, level='info')


def log2see(code, message):
    """Log message to file and STDOUT, but don't die.

    Args:
        code: Message code
        message: Message text

    Returns:
        None

    """
    # Log to screen and file
    _logit(code, message, verbose=True, error=False)


def log2die(code, message):
    """Log to STDOUT and file, then die.

    Args:
        code: Error number
        message: Descriptive error string

    Returns:
        None

    """
    _logit(code, message, error=True)


def log2exception_die(code, sys_exc_info, message=None):
    """Log trace message to file and STDOUT, but don't die.

    Args:
        code: Message code
        sys_exc_info: Tuple from exception from sys.exc_info
        message: Descriptive error string

    Returns:
        None

    """
    # Log
    log2exception(code, sys_exc_info, message=message, die=True)


def log2exception(code, sys_exc_info, message=None, die=False):
    """Log trace message to file and STDOUT, but don't die.

    Args:
        code: Message code
        sys_exc_info: Tuple from exception from sys.exc_info
        die: Die if True

    Returns:
        None

    """
    # Initialize key variables
    (exc_type, exc_value, exc_traceback) = sys_exc_info
    log_message = ('''\
[Bug: Exception Type:{}, Exception Instance: {}, Stack Trace Object: {}]\
'''.format(exc_type, exc_value, exc_traceback))
    log2warning(code, log_message)
    if bool(message) is True:
        log2warning(code, message)

    # Write trace to log file
    from pattoo_shared.configuration import BaseConfig
    config = BaseConfig()
    log_file = config.log_file()
    with open(log_file, 'a+') as _fh:
        traceback.print_tb(exc_traceback, file=_fh)

    # Die
    if bool(die) is True:
        log2die(code, log_message)


def _logit(error_num, error_string, error=False, verbose=False, level='info'):
    """Log errors to file and STDOUT.

    Args:
        error_num: Error number
        error_string: Descriptive error string
        error: Is this an error or not?
        verbose: If True print non errors to STDOUT
        level: Logging level

    Returns:
        None

    """
    # Define key variables
    global LOGGER
    username = getpass.getuser()
    levels = ['debug', 'info', 'warning', 'error', 'critical']

    # Set logging level
    if level in levels:
        log_level = level
    else:
        log_level = 'debug'

    # Create logger if it doesn't already exist
    if bool(LOGGER) is False:
        LOGGER = _GetLog()
    logger_file = LOGGER.logfile()
    logger_stdout = LOGGER.stdout()

    # Log the message
    if error:
        log_message = (
            '[{}] ({}E): {}'.format(
                username, error_num, error_string))
        logger_stdout.critical('%s', log_message)
        logger_file.critical(log_message)

        # All done
        sys.exit(2)
    else:
        log_message = (
            '[{}] ({}S): {}'.format(
                username, error_num, error_string))
        _logger_file(logger_file, log_message, log_level)
        if verbose:
            _logger_stdout(logger_stdout, log_message, log_level)


def _logger_file(logger_file, log_message, log_level):
    """Log to file at a particular logging level.

    Args:
        logger_file: File logger instance
        log_message: Logging message
        log_level: Logging level

    Returns:
        None

    """
    # Log accordingly
    if log_level == 'debug':
        logger_file.debug(log_message)
    elif log_level == 'info':
        logger_file.info(log_message)
    elif log_level == 'warning':
        logger_file.warning(log_message)
    elif log_level == 'error':
        logger_file.error(log_message)
    else:
        logger_file.critical(log_message)


def _logger_stdout(logger_stdout, log_message, log_level):
    """Log to stdout at a particular logging level.

    Args:
        logger_stdout: stdout logger instance
        log_message: Logging message
        log_level: Logging level

    Returns:
        None

    """
    # Log accordingly
    if log_level == 'debug':
        logger_stdout.debug(log_message)
    elif log_level == 'info':
        logger_stdout.info(log_message)
    elif log_level == 'warning':
        logger_stdout.warning(log_message)
    elif log_level == 'error':
        logger_stdout.error(log_message)
    else:
        logger_stdout.critical(log_message)


def _message(code, message, error=True):
    """Create a formatted message string.

    Args:
        code: Message code
        message: Message text
        error: If True, create a different message string

    Returns:
        output: Message result

    """
    # Initialize key variables
    time_object = datetime.datetime.fromtimestamp(time.time())
    timestring = time_object.strftime('%Y-%m-%d %H:%M:%S,%f')
    username = getpass.getuser()

    # Format string for error message, print and die
    if error is True:
        prefix = 'ERROR'
    else:
        prefix = 'STATUS'
    output = ('{} - {} - {} - [{}] {}'.format(
        timestring, username, prefix, code, message))

    # Return
    return output


def env():
    """Check enviroment variables before running scripts.

    Args:
        None

    Returns:
        None

    """
    # Make sure the PATTOO_CONFIGDIR environment variable is set
    if 'PATTOO_CONFIGDIR' not in os.environ:
        log_message = (
            'The PATTOO_CONFIGDIR environment variable not set.')
        log2die_safe(1039, log_message)

    # Make sure the PATTOO_CONFIGDIR environment variable is set to unittest
    if 'unittest' in os.environ['PATTOO_CONFIGDIR'].lower():
        log_message = ('''\
The PATTOO_CONFIGDIR is set to a unittest directory. Daemon cannot be run''')
        log2die_safe(1040, log_message)



#!/usr/bin/env python3
"""Test the daemon module."""

# Standard imports
import unittest
import os
import sys
import multiprocessing


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import agent, files
from pattoo_shared.configuration import Config
from pattoo_shared.variables import AgentAPIVariable
from tests.libraries.configuration import UnittestConfig
from tests.libraries import general

# Test Daemon importation
from tests.pattoo_shared_ import test_daemon


class TestAgent(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    config = Config()

    def test___init__(self):
        """Testing method or function named __init__."""
        # Initialize key variables
        parent = 'parent'
        child = 'child'

        # Test - No Child
        tester = agent.Agent(parent, config=self.config)
        self.assertEqual(tester.parent, parent)
        expected = files.pid_file(parent, self.config)
        self.assertEqual(tester.pidfile_parent, expected)
        expected = files.lock_file(parent, self.config)
        self.assertEqual(tester.lockfile_parent, expected)
        expected = files.pid_file(None, self.config)
        self.assertEqual(tester._pidfile_child, expected)

        # Test - With Child
        tester = agent.Agent(parent, child=child, config=self.config)
        self.assertEqual(tester.parent, parent)
        expected = files.pid_file(parent, self.config)
        self.assertEqual(tester.pidfile_parent, expected)
        expected = files.lock_file(parent, self.config)
        self.assertEqual(tester.lockfile_parent, expected)
        expected = files.pid_file(child, self.config)
        self.assertEqual(tester._pidfile_child, expected)

    def test_name(self):
        """Testing method or function named name."""
        # Initialize key variables
        parent = 'parent'

        # Test
        tester = agent.Agent(parent)
        self.assertEqual(tester.name(), parent)

    def test_run(self):
        """Testing method or function named query."""
        pass


class TestAgentDaemon(test_daemon.TestDaemon):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def setUp(self):
        """Test setup"""

        # Setup base config and agent
        self._config = Config()
        agent_name = general.random_agent_name()
        self._agent = agent.Agent(agent_name, config=self._config)

        # Instantiation of test daemon
        self._daemon = agent.AgentDaemon(self._agent)

    def test_run(self):
        """Testing method or function named run."""
        pass


class TestGracefulAgentDaemon(test_daemon.TestGracefulDaemon):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def setUp(self):
        """Test setup"""

        # Setup base config and agent
        self._config = Config()
        agent_name = general.random_agent_name()
        self._agent = agent.Agent(agent_name, config=self._config)

        # Instantiation of test daemon
        self._daemon = agent.GracefulAgentDaemon(self._agent)

    def test_run(self):
        """Testing method or function named run."""
        pass


class TestAgentCLI(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method or function named __init__."""
        pass

    def test_process(self):
        """Testing method or function named process."""
        pass

    def test_control(self):
        """Testing method or function named control."""
        pass


class TestAgentAPI(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method or function named __init__."""
        pass

    def test_query(self):
        """Testing method or function named query."""
        pass


class Test_StandaloneApplication(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method or function named __init__."""
        pass

    def test_load_config(self):
        """Testing method or function named load_config."""
        pass

    def test_load(self):
        """Testing method or function named load."""
        pass


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test__number_of_workers(self):
        """Testing method or function named _number_of_workers."""
        # Test
        expected = (multiprocessing.cpu_count() * 2) + 1
        result = agent._number_of_workers()
        self.assertEqual(result, expected)

    def test__ip_binding(self):
        """Testing method or function named _ip_binding."""
        # Test with Hostname
        ip_listen_address = 'localhost'
        ip_bind_port = '1000'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, ip_listen_address=ip_listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = 'localhost:1000'
        self.assertEqual(result, expected)

        # Test with IPv4
        ip_listen_address = '1.2.3.4'
        ip_bind_port = '5678'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, ip_listen_address=ip_listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = '1.2.3.4:5678'
        self.assertEqual(result, expected)

        # Test with IPv6
        ip_listen_address = '1::1'
        ip_bind_port = '91011'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, ip_listen_address=ip_listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = '[1::1]:91011'
        self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the times module."""

# Standard imports
import unittest
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import times
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################
    def test_validate_timestamp(self):
        """Testing function validate_timestamp."""
        # Initialize key variables
        result = times.validate_timestamp(300, 300)
        self.assertEqual(result, True)
        result = times.validate_timestamp(400, 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, '300')
        self.assertEqual(result, False)
        result = times.validate_timestamp('500', 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, 0)
        self.assertEqual(result, False)

    def test_normalized_timestamp(self):
        """Testing function normalized_timestamp."""
        # Initialize key variables
        polling_interval = 300

        timestamp = 31
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 0)

        timestamp = 301
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 300)

        timestamp = 900
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 900)

        timestamp = 1000
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 900)

        # Initialize key variables
        polling_interval = 30

        timestamp = 31
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 30)

        timestamp = 301
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 300)

        timestamp = 900
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 900)

        timestamp = 1000
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, 990)

        # Test with boolean None values
        timestamp = 900001
        for polling_interval in [None, [], False, {}]:
            result = times.normalized_timestamp(
                polling_interval, timestamp=timestamp)
            self.assertEqual(result, 900000)

        # Test with non integer values
        timestamp = 300
        for polling_interval in ['1', [1]]:
            with self.assertRaises(SystemExit):
                times.normalized_timestamp(
                    polling_interval, timestamp=timestamp)

    def test_timestamps(self):
        """Testing function timestamps."""
        # Test
        ts_start = 0
        ts_stop = 100
        polling_interval = 10
        expected = list(range(
            ts_start, ts_stop + polling_interval, polling_interval))
        result = times.timestamps(ts_start, ts_stop, polling_interval)
        self.assertEqual(result, expected)

        # Test with more irregular values
        ts_start = 11
        ts_stop = 73
        polling_interval = 13
        expected = [0, 13, 26, 39, 52, 65]
        result = times.timestamps(ts_start, ts_stop, polling_interval)
        self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the converter module."""

# Standard imports
import unittest
import os
import sys
from time import sleep

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import converter
from pattoo_shared.configuration import Config
from pattoo_shared.variables import (
    DataPointMetadata, DataPoint, TargetDataPoints, AgentPolledData)
from pattoo_shared.constants import (
    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE,
    DATAPOINT_KEYS, PattooDBrecord)
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    config = Config()

    def test_cache_to_keypairs(self):
        """Testing method or function named cache_to_keypairs."""
        cache = {
            'pattoo_agent_id': '123bb3a17c6cc915a98a859226d282b394ee0964956b7'
                               'd23c145fe9d94567241',
            'pattoo_agent_polling_interval': 10000,
            'pattoo_agent_timestamp': 1575789070210,
            'pattoo_datapoints': {
                'datapoint_pairs': [
                    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                    [11, 1, 2, 3, 4, 5, 6, 7, 12, 13, 14]],
                'key_value_pairs': {
                    '0': ['agent_snmpd_oid', '.1.3.6.1.2.1.2.2.1.10.1'],
                    '1': ['pattoo_agent_hostname', 'swim'],
                    '2': ['pattoo_agent_id',
                          '123bb3a17c6cc915a98a859226d282b'
                          '394ee0964956b7d23c145fe9d94567241'],
                    '3': ['pattoo_agent_polled_target', 'localhost'],
                    '4': ['pattoo_agent_polling_interval', '10000'],
                    '5': ['pattoo_agent_program', 'pattoo_agent_snmpd'],
                    '6': ['pattoo_key', 'agent_snmpd_.1.3.6.1.2.1.2.2.1.10'],
                    '7': ['pattoo_data_type', 32],
                    '8': ['pattoo_value', 4057039287.0],
                    '9': ['pattoo_timestamp', 1575789070107],
                    '10': ['pattoo_checksum',
                           'f6ac40a45ec4fa7d19823d8c318bb0c6d0a19f79ad4b04ab5'
                           '290d65f42250296'],
                    '11': ['agent_snmpd_oid', '.1.3.6.1.2.1.2.2.1.10.2'],
                    '12': ['pattoo_value', 0.0],
                    '13': ['pattoo_timestamp', 1575789070108],
                    '14': ['pattoo_checksum',
                           '2849bef00a05419c4614610814a1a8088a89a81a8192dbfa2'
                           '77598e1ff3389da']
                }
            }
        }

        # Test
        pattoo_db_records = converter.cache_to_keypairs(cache)

        # pattoo_db_records[0]
        # should have the values in [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        result = pattoo_db_records[0]
        expected = PattooDBrecord(
            pattoo_checksum='5306d34dac4c0c90b5cdc09c37a62c0d3056c6f58044b7a9'
                            'e2c93c65d0de9d88e4173b5972e73b59cecdad4a80e28fc7'
                            'a0edd32db653156b05cf80efb818f5c8',
            pattoo_metadata=[
                ('agent_snmpd_oid', '.1.3.6.1.2.1.2.2.1.10.1')],
            pattoo_data_type=32,
            pattoo_key='agent_snmpd_.1.3.6.1.2.1.2.2.1.10',
            pattoo_value=4057039287.0,
            pattoo_timestamp=1575789070107,
            pattoo_agent_polled_target='localhost',
            pattoo_agent_program='pattoo_agent_snmpd',
            pattoo_agent_hostname='swim',
            pattoo_agent_id='123bb3a17c6cc915a98a859226d282b394ee0964956b7d2'
                            '3c145fe9d94567241',
            pattoo_agent_polling_interval='10000')
        self.assertEqual(result, expected)

        # pattoo_db_records[1]
        # should have the values in [11, 1, 2, 3, 4, 5, 6, 7, 12, 13, 14]
        result = pattoo_db_records[1]
        expected = PattooDBrecord(
            pattoo_checksum='5decef67c60b4528f142373f2d796b28241c9af51d78abe5'
                            '29c92830e7491134de1dd600fb9c517f72a57a0f68640684'
                            '4a3ca1138c69a494cbedcb36e5e866b9',
            pattoo_metadata=[
                ('agent_snmpd_oid', '.1.3.6.1.2.1.2.2.1.10.2')],
            pattoo_data_type=32,
            pattoo_key='agent_snmpd_.1.3.6.1.2.1.2.2.1.10',
            pattoo_value=0.0,
            pattoo_timestamp=1575789070108,
            pattoo_agent_polled_target='localhost',
            pattoo_agent_program='pattoo_agent_snmpd',
            pattoo_agent_hostname='swim',
            pattoo_agent_id='123bb3a17c6cc915a98a859226d282b394ee0964956b7d23'
                            'c145fe9d94567241',
            pattoo_agent_polling_interval='10000')
        self.assertEqual(result, expected)

    def test__make_pattoo_db_record(self):
        """Testing method or function named _make_pattoo_db_record."""
        pass

    def test_agentdata_to_datapoints(self):
        """Testing method or function named agentdata_to_datapoints."""
        # Setup AgentPolledData
        agent_program = 'panda_bear'
        polling_interval = 20
        apd = AgentPolledData(agent_program, polling_interval)

        # Initialize TargetDataPoints
        target = 'teddy_bear'
        ddv = TargetDataPoints(target)

        # Setup DataPoint
        value = 457
        key = 'gummy_bear'
        data_type = DATA_INT
        variable = DataPoint(key, value, data_type=data_type)

        # Add data to TargetDataPoints
        ddv.add(variable)

        # Test TargetGateway to AgentPolledData
        apd.add(ddv)

        # Test contents
        expected_metadata = {
            'pattoo_agent_id': apd.agent_id,
            'pattoo_agent_program': agent_program,
            'pattoo_agent_hostname': apd.agent_hostname,
            'pattoo_agent_polled_target': target,
            'pattoo_agent_polling_interval': apd.agent_polling_interval
        }
        result = converter.agentdata_to_datapoints(apd)

        self.assertEqual(len(result), 1)
        item = result[0]
        self.assertTrue(isinstance(item, DataPoint))
        self.assertEqual(item.value, value)
        self.assertEqual(item.data_type, DATA_INT)
        self.assertEqual(item.key, key)
        self.assertTrue(isinstance(item.metadata, dict))
        self.assertEqual(len(item.metadata), len(expected_metadata))
        for key, value in item.metadata.items():
            self.assertTrue(isinstance(value, str))
            self.assertTrue(isinstance(key, str))
            self.assertEqual(value, str(expected_metadata[key]))

    def test_datapoints_to_dicts(self):
        """Testing method or function named datapoints_to_dicts."""
        # Initialize key variables
        datapoints = []

        # Create DataPoints
        for value in range(0, 2):
            # Sleep to force a change in the timestamp
            sleep(0.1)

            metadata = []
            for meta in range(0, 2):
                metadata.append(DataPointMetadata(int(meta), str(meta * 2)))

            # Create the datapoint
            datapoint = DataPoint(
                'label_{}'.format(value), value, data_type=DATA_INT
            )
            # Add metadata
            for meta in metadata:
                datapoint.add(meta)

            # Add metadata that should be ignored.
            for key in DATAPOINT_KEYS:
                metadata.append(DataPointMetadata(key, '_{}_'.format(key)))

            # Add the datapoint to the list
            datapoints.append(datapoint)

        # Start testing
        result = converter.datapoints_to_dicts(datapoints)
        expected = {
            'key_value_pairs': {
                0: ('0', '0'),
                1: ('1', '2'),
                2: ('pattoo_key', 'label_0'),
                3: ('pattoo_data_type', 99),
                4: ('pattoo_value', 0),
                5: ('pattoo_timestamp', 1575794447250),
                6: ('pattoo_checksum',
                    '284d21bff49bbde9eb7fc3ad98a88e5cbf72830f69743b47bd2c349'
                    '407807f68'),
                7: ('pattoo_key', 'label_1'),
                8: ('pattoo_value', 1),
                9: ('pattoo_timestamp', 1575915772433),
                10: ('pattoo_checksum',
                     'a5919eb5fc5bac62e7c80bc04155931f75e22166ed84b1d07f704f4'
                     '0b083d098')},
            'datapoint_pairs': [[0, 1, 2, 3, 4, 5, 6], [0, 1, 7, 3, 8, 9, 10]]}

        self.assertEqual(
            result['datapoint_pairs'], expected['datapoint_pairs'])
        for key, value in result['key_value_pairs'].items():
            if key not in [5, 9]:
                self.assertEqual(expected['key_value_pairs'][key], value)

    def test_agentdata_to_post(self):
        """Testing method or function named agentdata_to_post."""
        # Setup AgentPolledData
        agent_program = 'panda_bear'
        polling_interval = 20
        apd = AgentPolledData(agent_program, polling_interval)

        # Initialize TargetDataPoints
        target = 'teddy_bear'
        ddv = TargetDataPoints(target)

        # Setup DataPoint
        value = 457
        key = 'gummy_bear'
        data_type = DATA_INT
        variable = DataPoint(key, value, data_type=data_type)

        # Add data to TargetDataPoints
        ddv.add(variable)

        # Test TargetGateway to AgentPolledData
        apd.add(ddv)
        result = converter.agentdata_to_post(apd)
        self.assertEqual(result.pattoo_agent_id, apd.agent_id)
        self.assertEqual(
            result.pattoo_agent_polling_interval, polling_interval * 1000)
        self.assertTrue(isinstance(result.pattoo_datapoints, dict))

        # Test the key value pairs
        item = result.pattoo_datapoints['key_value_pairs']
        self.assertTrue('datapoint_pairs' in result.pattoo_datapoints)
        self.assertTrue('key_value_pairs' in result.pattoo_datapoints)
        self.assertTrue(isinstance(item, dict))

        # Convert item to a list of tuples for ease of testing
        tester = [(k, v) for k, v in sorted(item.items())]
        self.assertEqual(
            tester[0],
            (0, ('pattoo_agent_polling_interval', '20000')))

        self.assertEqual(
            tester[3:8],
            [
                (3, ('pattoo_agent_polled_target', 'teddy_bear')),
                (4, ('pattoo_agent_program', 'panda_bear')),
                (5, ('pattoo_key', 'gummy_bear')),
                (6, ('pattoo_data_type', 99)),
                (7, ('pattoo_value', 457))]
        )

        # Test the pointers to the key value pairs
        item = result.pattoo_datapoints['datapoint_pairs']
        self.assertTrue(isinstance(item, list))
        self.assertEqual(len(item), 1)
        self.assertEqual(len(item[0]), 10)

    def test_datapoints_to_post(self):
        """Testing method or function named datapoints_to_post."""
        # Initialize key variables
        key = '_key'
        value = '_value'
        datapoints = [DataPoint(key, value)]
        source = '1234'
        polling_interval = 20
        result = converter.datapoints_to_post(
            source, polling_interval, datapoints)

        # Test
        self.assertEqual(
            result.pattoo_agent_polling_interval, polling_interval)
        self.assertEqual(result.pattoo_agent_id, source)
        self.assertEqual(result.pattoo_datapoints, datapoints)
        self.assertEqual(result.pattoo_datapoints[0].key, key)
        self.assertEqual(result.pattoo_datapoints[0].value, value)

    def test_posting_data_points(self):
        """Testing method or function named posting_data_points."""
        # Initialize key variables
        key = '_key'
        value = '_value'
        datapoints = [DataPoint(key, value)]
        source = '1234'
        polling_interval = 20
        pdp = converter.datapoints_to_post(
            source, polling_interval, datapoints)
        result = converter.posting_data_points(pdp)

        # Test
        self.assertEqual(
            result['pattoo_agent_polling_interval'], polling_interval)
        self.assertEqual(result['pattoo_agent_id'], source)
        self.assertEqual(result['pattoo_datapoints'], datapoints)
        self.assertEqual(result['pattoo_datapoints'][0].key, key)
        self.assertEqual(result['pattoo_datapoints'][0].value, value)

    def test__keypairs(self):
        """Testing method or function named _keypairs."""
        # Test
        data = {'test this out': 7}
        result = converter._keypairs(data)
        expected = []
        self.assertEqual(result, expected)

        data = {'test this out': '7'}
        result = converter._keypairs(data)
        expected = [('test_this_out', '7')]
        self.assertEqual(result, expected)

    def test__checksum(self):
        """Testing method or function named _checksum."""
        # Test
        result = converter._checksum(1, 2, 3)
        expected = ('''\
3c9909afec25354d551dae21590bb26e38d53f2173b8d3dc3eee4c047e7ab1c1eb8b85103e3be7\
ba613b31bb5c9c36214dc9f14a42fd7a2fdb84856bca5c44c2''')
        self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the daemon module."""

# Standard imports
import unittest
import os
import subprocess
import sys
import shlex
from time import sleep


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import files
from pattoo_shared.daemon import Daemon, GracefulDaemon
from pattoo_shared.agent import Agent
from pattoo_shared.configuration import Config
from tests.libraries.configuration import UnittestConfig
from tests.libraries import general


def _create_agent():
    """Creates new test agent.

    Args:
        None

    Return:
        result: new agent for testing

    """
    # Return
    config = Config()
    name = general.random_agent_name()
    result = Agent(name, config=config)
    return result


def _script(_agent, action):
    """Start test daemon.

    Args:
        _agent: Agent object
        action: Action to take

    Return:
        result: Script to run

    """
    # Run script
    _path = os.path.join(ROOT_DIR, 'tests/bin/mock_daemon.py')
    result = 'python3 {} --{} --agent_name={}'.format(
        _path, action, _agent.name())
    return result


def _start(_agent):
    """Start test daemon.

    Args:
        _agent: Agent object

    Return:
        None

    """
    # Run script
    _daemonizer(_script(_agent, 'start'))


def _stop(_agent):
    """Stop test daemon.

    Args:
        _agent: Agent object

    Return:
        None

    """
    # Run script
    _daemonizer(_script(_agent, 'stop'))


def _restart(_agent):
    """Restart test daemon.

    Args:
        _agent: Agent object

    Return:
        None

    """
    # Run script
    _daemonizer(_script(_agent, 'restart'))


def _status(_agent):
    """Get daemon status.

    Args:
        _agent: Agent object

    Return:
        None

    """
    # Run script
    _daemonizer(_script(_agent, 'status'))


def _daemonizer(arguments):
    """Allows for the daemon start

    Args:
        arguments: CLI arguments

    Return:
        None

    """
    # Execute
    subprocess.call(shlex.split(arguments))


class _Run():
    """Class for creating a run method."""

    def run(self, loop=True):
        """Overriding Daemon run method

        Prints to standard output

        Args:
            loop: True when looping functionality is needed. Looping must be
                disabled when testing the run function, to avoid waiting for
                an infinite loop to end.

        Return:
            None

        """
        # Loop
        while loop:
            sleep(60)
        return True


class MockGracefulDaemon(_Run, GracefulDaemon):
    """Class to create graceful daemon for testing."""

    def __init__(self, _agent):
        """Initialize the class.

        Args:
            _agent: Agent

        Returns:
            None

        """
        # Setting up MockDaemon and starting process for testing
        GracefulDaemon.__init__(self, _agent)


class MockDaemon(_Run, Daemon):
    """Class to create daemon for testing."""

    def __init__(self, _agent):
        """Initialize the class.

        Args:
            _agent: Agent

        Returns:
            None

        """
        # Setting up MockDaemon and starting process for testing
        Daemon.__init__(self, _agent)


class TestDaemon(unittest.TestCase):
    """Test all Daemon class methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def setUp(self):
        """Test setup"""
        # Setup base config and agent
        self._config = Config()
        self._agent = _create_agent()

        # Instantiation of test daemon
        self._daemon = MockDaemon(self._agent)

    def tearDown(self):
        """Test clean up"""
        _stop(self._agent)

    def test___init__(self):
        """Testing function __init__."""
        # Check daemon name matches agent name
        self.assertEqual(self._daemon.name, self._agent.name())

        # Checking daemon pid_file
        expected = files.pid_file(self._agent.name(), self._config)
        self.assertEqual(self._daemon.pidfile, expected)

        # Checking daemon lock_file
        expected = files.lock_file(self._agent.name(), self._config)
        self.assertEqual(self._daemon.lockfile, expected)

    def test__daemonize(self):
        """Testing function _daemonize."""
        pass

    def test_delpid(self):
        """Testing function delpid."""

        # Creating daemon Process ID file(pidfile)
        os.mknod(self._daemon.pidfile)

        # Checking that pid file has been created
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Delete pid file
        self._daemon.delpid()

        # Check that pidfile of the daemon has been deleted
        self.assertFalse(os.path.exists(self._daemon.pidfile))

    def test_dellock(self):
        """Testing function dellock."""

        # Creating daemon Process ID file(pidfile)
        os.mknod(self._daemon.lockfile)

        # Checking that pid file has been created
        self.assertTrue(os.path.exists(self._daemon.lockfile))

        # Delete pid file
        self._daemon.dellock()

        # Check that pidfile of the daemon has been deleted
        self.assertFalse(os.path.exists(self._daemon.lockfile))

    def test_start(self):
        """Testing function start."""
        # Test starting
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        _start(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

    def test_force(self):
        """Testing function force."""
        # Test starting
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        _start(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Calling force stop
        self._daemon.force()
        self.assertFalse(os.path.exists(self._daemon.pidfile))

    def test_stop(self):
        """Testing function stop."""
        # Test starting
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        _start(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Calling stop
        os.mknod(self._daemon.lockfile)
        self._daemon.stop()
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        self.assertFalse(os.path.exists(self._daemon.lockfile))

    def test_restart(self):
        """Testing function restart."""
        # Initialize key variables
        sleep_time = 0.1

        # Test restarting from stop
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        _restart(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Sleep
        _start = os.stat(self._daemon.pidfile).st_mtime
        sleep(sleep_time)

        # Test restarting from start
        _restart(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Test age of PID file
        _stop = os.stat(self._daemon.pidfile).st_mtime
        self.assertTrue(_stop > _start)
        self.assertTrue(_stop - _start >= sleep_time)

    def test_status(self):
        """Testing function status."""
        # Test starting
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        _start(self._agent)
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Test stopping
        # Test start
        result = self._daemon.status()
        self.assertTrue(result)

        _stop(self._agent)
        self.assertFalse(os.path.exists(self._daemon.pidfile))

        # Test start
        result = self._daemon.status()
        self.assertFalse(result)

    def test_run(self):
        """Testing function run."""
        # Test
        result = self._daemon.run(loop=False)
        self.assertTrue(result)


class TestGracefulDaemon(TestDaemon):
    """Test all GracefulDaemon class methods."""

    def setUp(self):
        """Test setup"""
        # Setup base config and agent
        self._config = Config()
        self._agent = _create_agent()

        # Instantiation of test daemon
        self._daemon = MockGracefulDaemon(self._agent)

    def graceful_fn(self, callback):
        """Set up and execute test callback to implement graceful shutdown.

        Args:
            callback: function that implements graceful shutdown functionality

        Return:
            wrapper: Implements setup before using callback and making
            assertions

        """
        def wrapper():
            """Wrapper function to be returned by graceful_fn"""
            # Testing proper graceful shutdown by creating lock file to
            # simulate that a process is currently handling data.
            _start(self._agent)
            os.mknod(self._daemon.lockfile)
            self.assertTrue(os.path.exists(self._daemon.lockfile))
            callback()

            # Checking that both daemon pidfile and lockfile do not exist,
            # which indicates successful stoppage of daemon.
            self.assertFalse(os.path.exists(self._daemon.lockfile))
            self.assertFalse(os.path.exists(self._daemon.pidfile))

        return wrapper

    def test_stop(self):
        """Testing graceful stop function"""
        # Test base Daemon stop functionality
        # When lock file does not exist
        super(TestGracefulDaemon, self).test_stop()

        # Graceful stop testing
        self.graceful_fn(self._daemon.stop())

    def test_restart(self):
        """Testing graceful restart function"""

        # Test base Daemon restart functionality
        # When lock file does not exist
        super(TestGracefulDaemon, self).test_restart()

        # Graceful stop testing
        self.graceful_fn(_restart(self._agent))


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}pattoo_shared_'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import network
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_get_ipaddress(self):
        """Testing method or function named get_ip_address."""
        # Test bad addresses
        bad_addresses = [123, 'koala_bear', None, True, False, {}, []]
        for item in bad_addresses:
            result = network.get_ipaddress(item)
            self.assertIsNone(result)

        # Test good addresses
        good_addresses = ['127.0.0.1', '::1', 'localhost', 'www.google.com']
        for item in good_addresses:
            result = network.get_ipaddress(item)
            self.assertTrue(bool(result))


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



"""Module used to manage network resources."""

# Import PIP libraries
import ipaddress
import socket


def get_ipaddress(device):
    """Get IP address for a device.

    Args:
        device: Device for which address is required

    Returns:
        result: IP address

    """
    # Initialize key variables
    result = None

    # ipaddress thinks integer values are IPv4 addresses
    if isinstance(device, int) is False:
        # Check if IP address
        try:
            _ = ipaddress.ip_address(device)
            result = device
        except:
            # Not an IP address
            try:
                result = socket.gethostbyname(device)
            except:
                result = None

    # Return
    return result



"""Functions for installing external packages."""
# Standard imports
import os
import getpass
import re
from collections import namedtuple

# Import pattoo related libraries
from pattoo_shared.installation import shared
from pattoo_shared.installation import environment
from pattoo_shared import log


def installed_version(package):
    """Retrieve installed pip package version.

    Args:
        package: The name of the package

    Returns:
        version: The version of the package if the package is installed.
        None: If the package is not installed

    """
    # Initialize key variable
    version = None
    output = ''

    # Get desired package name and version
    details = package_details(package)

    # Get version from OS
    try:
        (_, output, __) = shared.run_script(
            'python3 -m pip show {}'.format(details.name),
            verbose=False)
    except:
        return version

    # Process data
    lines = output.decode().strip().split('\n')
    if bool(lines):
        version = lines[1].split(':')[1].strip()
    return version


def install_package(package, verbose=False):
    """Automatically Install missing pip3 packages.

    Args:
        package: The pip3 package to be installed

    Returns:
        None

    """
    # Initialize key variables
    command = (
        'python3 -m pip install --force-reinstall {0}'.format(package))

    # Get desired package name and version
    details = package_details(package)

    # Return if there is no package
    if bool(details.name) is False:
        return

    # Get installed version
    current_version = installed_version(package)
    already_installed = bool(current_version)

    # Update if there is a '<' or '>' in the desired package.
    if already_installed is True:
        # Do nothing if installed and proposed versions match
        if (current_version == details.version) and bool(
                details.inequality) is False:
            return
        # Do nothing if no version is specified
        elif bool(details.version) is False:
            return

    # Install
    try:
        shared.run_script(command, verbose=verbose)
    except SystemExit:
        message = 'Invalid pip package/package version "{}"'.format(package)
        log.log2die_safe(1088, message)


def install(requirements_dir, install_dir, verbose=False):
    """Ensure PIP3 packages are installed correctly.

    Args:
        requirements_dir: The directory with the pip_requirements file.
        install_dir: Directory where packages must be installed.
        verbose: Print status messages if True

    Returns:
        True if pip3 packages are installed successfully

    """
    # Initialize key variables
    lines = []
    filepath = '{}{}pip_requirements.txt'.format(requirements_dir, os.sep)
    path = environment.PIPpath('{}{}bin'.format(install_dir, os.sep))

    # Say what we are doing
    print('Checking pip3 packages')
    if os.path.isfile(filepath) is False:
        shared.log('Cannot find PIP3 requirements file {}'.format(filepath))

    # Opens pip_requirements file for reading
    try:
        _fp = open(filepath, 'r')
    except PermissionError:
        log.log2die_safe(1079, '''\
Insufficient permissions for reading the file: {}. \
Ensure the file has read-write permissions and try again'''.format(filepath))
    else:
        with _fp:
            lines = _fp.readlines()

    # Strip unprintable characters from ends
    lines = [_.strip() for _ in lines]

    # Remove comments and blank lines
    packages = [
        _ for _ in lines if (_.startswith('#') is False) and (bool(_) is True)]

    # Process each line of the file
    for package in packages:
        # We need to pre-pend the install_dir to the OS path for venv version
        # of Python3 pip to correctly identify packages installed in the
        # venv environment
        path.set()
        install_package(package, verbose=verbose)
        path.reset()

    # Set ownership of any newly installed python packages to pattoo user
    if getpass.getuser() == 'root':
        if os.path.isdir(install_dir) is True:
            shared.run_script('chown -R pattoo:pattoo {}'.format(
                install_dir), verbose=verbose)

    print('pip3 packages successfully installed')


def package_details(package_):
    """Get package details.

    Args:
        package_: The pip3 package to be installed

    Returns:
        result: Named tuple of package name and version

    """
    # Initialize key variables
    package = ''.join(package_.split())
    Package = namedtuple('Package', 'name version inequality')
    ideal_length = 3
    inequalities = ['=', '<', '>', '~']
    inequality = False

    # Get desired package name and version
    nodes = re.split('|'.join(inequalities), package)
    nodes.extend([None] * (ideal_length - len(nodes)))
    name = nodes[0]
    version = nodes[2]

    # Determine whether there is an inequality in the string
    for item in inequalities:
        if item in package:
            inequality = True

    # Return
    result = Package(name=name, version=version, inequality=inequality)
    return result
